// ----
// ---- file   : SyConnection.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2090, 
// ----          22May2010, 26May2010, 27May2010, 26Oct2010
// ----
// ----
// ----

#include "tksynth.h"


SyConnection::SyConnection(void) {
   next      = NULL;
   module    = NULL;
   output_id = 0;
   level     = 1.0f;
   offset    = 0.0f;

   min_level = 0.0f;
   max_level = 2.0f;

   min_offset = -1.0f;
   max_offset = 1.0f;

   ui_position_x = 0.0f;
   ui_position_y = 0.0f;

   ui_autolayout = 0;

   ui_object = NULL;
}

SyConnection::~SyConnection() {
}

void SyConnection::setLevel(sF32 _level) {
   level = _level;
}

sF32 SyConnection::getLevel(void) {
   return level;
}

void SyConnection::setOffset(sF32 _offset) {
   offset = _offset;
}

sF32 SyConnection::getOffset(void) {
   return offset;
}

void SyConnection::setMinLevel(sF32 _min) {
   min_level = _min;
}

sF32 SyConnection::getMinLevel(void) {
   return min_level;
}

void SyConnection::setMaxLevel(sF32 _max) {
   max_level = _max;
}

sF32 SyConnection::getMaxLevel(void) {
   return max_level;
}

void SyConnection::setMinOffset(sF32 _min) {
   min_offset = _min;
}

sF32 SyConnection::getMinOffset(void) {
   return min_offset;
}

void SyConnection::setMaxOffset(sF32 _max) {
   max_offset = _max;
}

sF32 SyConnection::getMaxOffset(void) {
   return max_offset;
}


YAC_Object *SyConnection::getModule(void) {
   return module;
}

sUI SyConnection::getOutputId(void) {
   return output_id;
}

void SyConnection::replaceSource(YAC_Object *_mod, sUI _outId) {
   if(YAC_BCHK(_mod, clid_SyModule))
   {
      module    = (SyModule*) _mod;
      output_id = _outId;
   }
   else
   {
      // d'oh
      module    = NULL;
      output_id = 0;
   }
}

void SyConnection::setUiPositionX(sF32 _x) {
   ui_position_x = _x;
}

sF32 SyConnection::getUiPositionX(void) {
   return ui_position_x;
}

void SyConnection::setUiPositionY(sF32 _y) {
   ui_position_y = _y;
}

sF32 SyConnection::getUiPositionY(void) {
   return ui_position_y;
}

void SyConnection::setUiObject(YAC_Object *_o) {
   ui_object = _o;
}

YAC_Object *SyConnection::getUiObject(void) {
   return ui_object;
}

void SyConnection::setUiAutoLayout(sBool _bEnabled) {
   ui_autolayout = _bEnabled;
}

sBool SyConnection::getUiAutoLayout(void) {
   return ui_autolayout;
}
