// ----
// ---- file   : SyConnection.h
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2010, 
// ----          22May2010, 27May2010, 26Oct2010
// ----
// ----
// ----

#ifndef __TKSYNTH_CONNECTION_H__
#define __TKSYNTH_CONNECTION_H__

YG("tksynth");


class SyModule;


YCR class SyConnection : public YAC_Object {
  public:
   SyConnection *next;

   SyModule *module; // connection source module reference

   sU16 output_id;   // source module output id
   sF32 level;       // output value scaling factor
   sF32 offset;      // output value constant offset

   sF32 min_level;  // suggested value (*not* clamped, just a hint for the GUI)
   sF32 max_level;  // ^^
   sF32 min_offset; // ^^ 
   sF32 max_offset; // ^^

   sF32 ui_position_x; // position of UI control point
   sF32 ui_position_y;

   sBool ui_autolayout; // 1=auto-layout connection, 0=connection is placed manually

   YAC_Object *ui_object; // reference to UI display object (ctlpoint)

  public:
   YAC_POOLED(SyConnection, YAC_POOL_PRIORITY_LOW);

   SyConnection(void);
   ~SyConnection();

  public:
   YM void setLevel (sF32 _level);
   YM sF32 getLevel (void);

   YM void setOffset (sF32 _offset);
   YM sF32 getOffset (void);

   YM void setMinLevel (sF32 _min);
   YM sF32 getMinLevel (void);

   YM void setMaxLevel (sF32 _max);
   YM sF32 getMaxLevel (void);

   YM void setMinOffset (sF32 _min);
   YM sF32 getMinOffset (void);

   YM void setMaxOffset (sF32 _max);
   YM sF32 getMaxOffset (void);

   YM YAC_Object *getModule   (void);
   YM sUI         getOutputId (void);

   YM void        replaceSource (YAC_Object *_newMod, sUI _outId);

   YM void setUiPositionX (sF32 _x);
   YM sF32 getUiPositionX (void);

   YM void setUiPositionY (sF32 _y);
   YM sF32 getUiPositionY (void);

   YM void        setUiObject (YAC_Object *_o);
   YM YAC_Object *getUiObject (void);
   
   YM void  setUiAutoLayout (sBool _bEnabled);
   YM sBool getUiAutoLayout (void);
};

#define _SyConnection SyConnection


#endif // __TKSYNTH_CONNECTION_H__
