// ----
// ---- file   : SyInput.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2090, 
// ----          22May2010, 30Sep2010, 07Oct2010
// ----
// ----
// ----

#include "tksynth.h"


SyInput::SyInput(void) {
   connections     = NULL;
   reset_value     = 0.0f;
   min_reset_value = 0.0f;
   max_reset_value = 127.0f;

   interpol_amount     = 1.0f;
   min_interpol_amount = 0.0f;
   max_interpol_amount = 1.0f;

   Dsyloopallch()
   {
      last_value[chIdx] = 0.0f;
   }

   clamp_mode = SYINPUT_CLAMP_NONE;

   b_force_sampleexact_mode = 0;
}

SyInput::~SyInput() {
   freeConnections();
}

void SyInput::freeConnections(void) {
   SyConnection *c = connections;
   while(c)
   {
      SyConnection *n =c->next;
      YAC_DELETE(c);
      c = n;
   }
   connections = NULL;
}

sUI SyInput::getNumConnections(void) {
   sUI r = 0;
   SyConnection *c = connections;
   while(c)
   {
      r++;
      c = c->next;
   }
   return r;
}

YAC_Object *SyInput::getConnectionById(sUI _id) {
   SyConnection *c = connections;
   sUI i = 0;
   while(c)
   {
      if(i == _id)
      {
         return c;
      }
      i++;
      c = c->next;
   }
   return NULL;
}

void SyInput::setResetValue(sF32 _val) {
   reset_value = _val;

   ////printf("xxx SyInput::setResetValue: val=%f\n",_val);

   // If there are no connections, immediately copy reset value to current value
   //  (i.e. when dialing in a new value in the UI)
   if(NULL == connections)
   {
      // Initialize for feedback mode
      Dsyloopallch()
      {
         last_value[chIdx]    = reset_value;
         current_value[chIdx] = reset_value;
         ////printf("xxx SyInput::setResetValue: chIdx=%d\n", chIdx);
      }

      // Initialize for non-feedback mode
      sUI i;
      for(i=0; i<(SYPORT_MAXCHANNELS * SYPORT_MAXBUFSIZE); i++)
      {
         buffer[i] = reset_value * level + offset;
      }

   }

}

sF32 SyInput::getResetValue(void) {
   return reset_value;
}

void SyInput::setMinResetValue(sF32 _val) {
   min_reset_value = _val;
}

sF32 SyInput::getMinResetValue(void) {
   return min_reset_value;
}

void SyInput::setMaxResetValue(sF32 _val) {
   max_reset_value = _val;
}

sF32 SyInput::getMaxResetValue(void) {
   return max_reset_value;
}

void SyInput::setInterpol(sF32 _val) {
   if( (_val > 0.0f) && (_val <= 1.0f) )
   {
      interpol_amount = _val;
   }
   else
   {
      interpol_amount = 1.0f;
   }
}

sF32 SyInput::getInterpol(void) {
   return interpol_amount;
}

void SyInput::setMinInterpol(sF32 _val) {
   min_interpol_amount = _val;
}

sF32 SyInput::getMinInterpol(void) {
   return min_interpol_amount;
}

void SyInput::setMaxInterpol(sF32 _val) {
   max_interpol_amount = _val;
}

sF32 SyInput::getMaxInterpol(void) {
   return max_interpol_amount;
}

void SyInput::setClampMode(sSI _mode) {
   clamp_mode = _mode;
}

sSI SyInput::getClampMode(void) {
   return clamp_mode;
}

YAC_Object *SyInput::connect(YAC_Object *_module, sUI _moduleOutputId) {
   if(YAC_BCHK(_module, clid_SyModule))
   {
      SyModule *mod = (SyModule*) _module;

      if(_moduleOutputId < mod->num_outputs)
      {
         SyConnection *conn = (SyConnection*) YAC_NEW_POOLED(SyConnection);

         conn->module    = mod;
         conn->output_id = (sU16) _moduleOutputId;
         conn->next      = connections;

         connections = conn;

         return conn;
      }
   }
   return NULL;
}

void SyInput::disconnect(YAC_Object *_connection) {
   if(YAC_BCHK(_connection, clid_SyConnection))
   {
      SyConnection *conn = (SyConnection*) _connection;

      // find connection
      SyConnection *c = connections;
      if(c == conn)
      {
         // remove head
         connections = c->next;
         YAC_DELETE(c);
         return;
      }
      else
      {
         while(c->next)
         {
            if(conn == c->next)
            {
               SyConnection *n = c->next;
               c->next = n->next;
               YAC_DELETE(n);
               return;
            }
            c = c->next;
         }
         // not found
      }
   }
}

void SyInput::setEnableForceSampleExactMode(sSI _bEnabled) {
   b_force_sampleexact_mode = _bEnabled;
}

sSI SyInput::getEnableForceSampleExactMode(void) {
   return b_force_sampleexact_mode;
}
