// ----
// ---- file   : SyInput.h
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2090, 
// ----          22May2010, 01Jul2010, 24Sep2010, 07Oct2010
// ----
// ----
// ----

#ifndef __TKSYNTH_INPUT_H__
#define __TKSYNTH_INPUT_H__

YG("tksynth");

class SyConnection;


YCR class SyInput : public SyPort {
  public:
#define SYINPUT_CLAMP_NONE YCI 0
#define SYINPUT_CLAMP_CLIP YCI 1
#define SYINPUT_CLAMP_TANH YCI 2
#define SYINPUT_CLAMP_TRIG YCI 3
#define SYINPUT_CLAMP_TANH2 YCI 4

  public:
   SyConnection *connections;

   sF32 reset_value;
   sF32 min_reset_value; // determines range of (reset) value
   sF32 max_reset_value; // 

   sF32 last_value[SYPORT_MAXCHANNELS];          // last calculated value
   sF32 interpol_amount;     // 1.0=use new values immediately, 0.25=new value reached after 4 samples 
   sF32 min_interpol_amount; // gui hint
   sF32 max_interpol_amount; // gui hint

   sSI clamp_mode; // see SYINPUT_CLAMP_xxx

   sBool b_force_sampleexact_mode; // if 1, override port mode

  public:
   YAC_POOLED(SyInput, YAC_POOL_PRIORITY_LOW);

   SyInput(void);
   ~SyInput();
   
   void freeConnections(void);

   YM void setResetValue (sF32 _val);
   YM sF32 getResetValue (void);
   YM void setMinResetValue (sF32 _val);
   YM sF32 getMinResetValue (void);
   YM void setMaxResetValue (sF32 _val);
   YM sF32 getMaxResetValue (void);

   YM void setClampMode (sSI _mode);
   YM sSI  getClampMode (void);

   YM void setInterpol (sF32 _amount);
   YM sF32 getInterpol (void);
   YM void setMinInterpol (sF32 _amount);
   YM sF32 getMinInterpol (void);
   YM void setMaxInterpol (sF32 _amount);
   YM sF32 getMaxInterpol (void);


   // return SyConnection or null
   YM virtual YAC_Object *connect (YAC_Object *_module, sUI _moduleOutputId);

   YM virtual void disconnect(YAC_Object *_connection);

   YM sUI         getNumConnections (void);
   YM YAC_Object *getConnectionById (sUI _id);

   YM void setEnableForceSampleExactMode (sSI _bEnabled);
   YM sSI getEnableForceSampleExactMode (void);

};

#define _SyInput SyInput


#endif // __TKSYNTH_INPUT_H__
