// ----
// ---- file   : SyModule.h
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2012 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2009, 
// ----          22May2010, 03Jun2010, 30Sep2010, 06Oct2010, 07Oct2010, 23Jan2011, 05Feb2012
// ----
// ----
// ----

#ifndef __TKSYNTH_MODULE_H__
#define __TKSYNTH_MODULE_H__

YG("tksynth");

////class SyPatch;
class SyModule;


YCR class SyModule : public YAC_Object {
  public:
   static sSI next_tick_count; // used by output modules

  public:
   ////SyPatch *patch;  // reference to parent patch
   sSI  ui_instance_id; // for UI object to synth object mapping. upper 16bit is struct id, lower 16bit is module id
   sF32 ui_position_x;
   sF32 ui_position_y;
   sSI  ui_layout_style; // layout of inputs/outputs

   /////sSI calc_tick_count;  // used to determine whether the current module output state is valid
   sSI clock_tick_count; // -1=undefined, -2=reset

   SyInput ** inputs;  // getNumInputs() elements
   SyOutput **outputs; // getNumOutputs() elements

   sUI num_inputs;
   sUI num_outputs;

   sBool b_force_sampleexact_mode;

   YAC_String *name;

   sF32 mix_rate;

  public:
   static sF32 NoteToFreq(sF32 *_freqTable, sF32 _note);

  public:
   YAC_POOLED(SyModule, YAC_POOL_PRIORITY_LOW);

   SyModule(void);
   ~SyModule();

  protected:
   SyInput  *initInput  (sUI _idx, const char *_name, sUI _mode, sF32 _minResetValue, sF32 _maxResetValue, sF32 _resetValue);
   SyOutput *initOutput (sUI _idx, const char *_name, sUI _mode);

  public:
   void     freeInputs  (void);
   void     allocInputs (sUI _num);
   SyInput *addInput    (const char *_name, sUI _mode, sF32 _minResetValue, sF32 _maxResetValue, sF32 _resetValue);

   void      allocOutputs (sUI _num);
   SyOutput *addOutput    (const char *_name, sUI _mode);
   void      freeOutputs  (void);

   YM virtual void        setName (YAC_Object *_name);
   YM virtual YAC_Object *getName (void);
  
   YM void setUiInstanceId (sSI _id); // for UI
   YM sSI  getUiInstanceId (void);

   YM void setUiPositionX (sF32 _x);
   YM sF32 getUiPositionX (void);

   YM void setUiPositionY (sF32 _y);
   YM sF32 getUiPositionY (void);

   YM void setUiLayoutStyle (sSI _style);
   YM sSI  getUiLayoutStyle (void);


   YM virtual sUI getNumInputs     (void);
   YM YAC_Object *getInputById     (sUI _id);
   YM YAC_Object *getInputByName   (YAC_String *_name);
   YM sSI         getInputIdByName (YAC_String *_name);

   YM virtual sUI getNumOutputs     (void);
   YM YAC_Object *getOutputById     (sUI _id);
   YM YAC_Object *getOutputByName   (YAC_String *_name);
   YM sSI         getOutputIdByName (YAC_String *_name);

   // Initialize module, add inputs/outputs
   YM virtual void init(void);

   // Reset inputs and module state
   YM virtual void reset(void);

   // Evaluate source connections and calculate (single mono/stereo sample) output values
   //  This is called when the patch is running in feedback-capable mode
   //  (sample-exact evaluation of inputs / calculation of outputs)
   YM void calc (sSI _tickCount, sUI _framesAdvance);

   //
   // This method *MUST* be implemented by derived modules.
   //  Its purpose is to actually calculate the output values
   //
   virtual void doCalc(sUI _framesAdvance);

   // Evaluate source connections and calculate (single mono/stereo sample) output values
   //  This is called when the patch is running in non-feedback mode
   //  (block-exact evaluation of inputs / calculation of outputs)
   //
   //  If b_force_sampleexact_mode is true, the module's doCalc() function will be called, otherwise doCalcBlock()
   YM void calcBlock (sSI _tickCount, sUI _numFrames);

   //
   // This method should be implemented by derived modules.
   //  Its purpose is to actually calculate the output buffers
   //
   //  The fallback implementation will simply revert to calc()
   //
   virtual void doCalcBlock (sUI _numFrames);

   // Save static module configuration to stream
   YM virtual sBool saveStream (YAC_Object *_ofs);

   // Read static module configuration from stream
   YM virtual sBool readStream (YAC_Object *_ifs);

   YM void setEnableForceSampleExactMode (sSI _bEnabled);
   YM sSI  getEnableForceSampleExactMode (void);

   YM void setMixRate (sF32 _mixRate);
   YM sF32 getMixRate (void);

   YM virtual void setTickDuration (sF32 _tickDuration);

};

#define _SyModule SyModule


#endif // __TKSYNTH_MODULE_H__
