// ----
// ---- file   : SyModuleChnStereoJoin.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2013 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 28May2013
// ----
// ----
// ----

#include "tksynth.h"

#include "SyModuleChnStereoJoin.h"


void SyModuleChnStereoJoin::init(void) {
   SyModule::init();

   allocInputs(4);
   addInput("smp_l",   SYPORT_MODE_SE, -1.0f,  1.0f, 0.0f);  // #0
   addInput("smp_r",   SYPORT_MODE_SE, -1.0f,  1.0f, 0.0f);  // #1
   addInput("pan_l",   SYPORT_MODE_BE, -1.0f,  1.0f, -1.0f); // #2
   addInput("pan_r",   SYPORT_MODE_BE, -1.0f,  1.0f,  1.0f); // #3

   allocOutputs(1);
   addOutput("smp", SYPORT_MODE_SE); // #0
}

void SyModuleChnStereoJoin::reset(void) {
   SyModule::reset();
}

void SyModuleChnStereoJoin::doCalc(sUI _framesAdvance) {

   SyInput *in_smp_l  = inputs[0];
   SyInput *in_smp_r  = inputs[1];
   SyInput *in_pan_l  = inputs[2];
   SyInput *in_pan_r  = inputs[3];

   SyOutput *out_smp = outputs[0];

   sF32 smpL = Dsymono(in_smp_l);
   sF32 smpR = Dsymono(in_smp_r);

   sF32 pL = ((1.0f - Dsyctl(in_pan_l)) * 0.5f);
   sF32 pR = ((1.0f - Dsyctl(in_pan_r)) * 0.5f);

   sF32 outL = (smpL * pL)          + (smpR * pR);
   sF32 outR = (smpL * (1.0f - pL)) + (smpR * (1.0f - pR));

   // Scale/offset output samples and copy to output port
   Dsyoutleft (out_smp, outL);
   Dsyoutright(out_smp, outR);
}
