// ----
// ---- file   : SyModuleEQ3.h
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2090, 
// ----          22May2010, 07Oct2010, 12Oct2010
// ----
// ----
// ----

// based on:
//---------------------------------------------------------------------------
//
// 3 Band EQ :)
//
// EQ.H - Header file for 3 band EQ
//
// (c) Neil C / Etanza Systems / 2K6
//
// Shouts / Loves / Moans = etanza at lycos dot co dot uk
//
// This work is hereby placed in the public domain for all purposes, including
// use in commercial applications.
//
// The author assumes NO RESPONSIBILITY for any problems caused by the use of
// this software.
//
//----------------------------------------------------------------------------
// NOTES :
//
// - Original filter code by Paul Kellet (musicdsp.pdf)
//
// - Uses 4 first order filters in series, should give 24dB per octave
//
// - Now with P4 Denormal fix :)


//----------------------------------------------------------------------------

#ifndef __TKSYNTH_MODULE_EQ_3_H__
#define __TKSYNTH_MODULE_EQ_3_H__

YG("tksynth");

class SyModuleEQ3State {
  public:
   // Filter #1 (Low band)
   sF64 f1p0; // Poles ...
   sF64 f1p1;
   sF64 f1p2;
   sF64 f1p3;
      
   // Filter #2 (High band)
   sF64 f2p0; // Poles ...
   sF64 f2p1;
   sF64 f2p2;
   sF64 f2p3;
      
   // Sample history buffer
   sF64 sdm1; // Sample data minus 1
   sF64 sdm2; // 2
   sF64 sdm3; // 3

  public:

   inline void reset(void) {
      sdm1 = sdm2 = sdm3 = 0.0f;
   }

};


YC class SyModuleEQ3 : public SyModule {

   SyModuleEQ3State state[SYPORT_MAXCHANNELS];
      
  public:
   YAC_POOLED(SyModuleEQ3, YAC_POOL_PRIORITY_LOW);

   void init        (void);
   void reset       (void);
   void doCalc      (sUI _framesAdvance);
   void doCalcBlock (sUI _numFrames);
};


#endif // __TKSYNTH_MODULE_EQ_3_H__
