// ----
// ---- file   : SyModuleExtSample.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 13Jun2010, 30Sep2010, 07Oct2010
// ----          
// ----
// ----
// ----

#include "tksynth.h"

#include "SyModuleExtSample.h"


void SyModuleExtSample::init(void) {
   SyModule::init();

   //allocInputs(0);

   allocOutputs(1);

   addOutput("smp", SYPORT_MODE_SE);  // #0

   SyOutput *outSmp = (SyOutput *) getOutputById(0);
   outSmp->setNumChannels(2);

   ui_audio_generator = NULL;
}

void SyModuleExtSample::reset(void) {
   SyModule::reset();

   input_samples = NULL;
   input_offset  = 0;
}

void SyModuleExtSample::doCalc(sUI _framesAdvance) {

   SyOutput *output = outputs[0];

   // printf("xxx SyModuleExtSample::calc\n");

   if(NULL != input_samples)
   {
      // Output current l/r sample pair
      sUI chIdx = 0;
      sF32 *a = &input_samples[input_offset * 2]; // calc current sample pointer (stereo sample pairs)
      for(chIdx=0; chIdx < 2; chIdx++)
      {
         sF32 s = a[chIdx];
         ////printf("xxx out s=%f\n", s);
         Dsyout(output, s);
      }
   }
   else
   {
      // No input samples, play silence
      sUI chIdx = 0;
      for(chIdx=0; chIdx < 2; chIdx++)
      {
         Dsyout(output, 0.0f);
      }
   }


   // tick
   ////printf("xxx SyModuleExtSample::tick\n");
   input_offset++;

   if(input_offset >= num_samples)
   {
      input_offset = 0;
   }

}

void SyModuleExtSample::setSamples(YAC_Object *_fa) {
   if(YAC_BCHK(_fa, YAC_CLID_FLOATARRAY))
   {
      YAC_CAST_ARG(YAC_FloatArray, fa, _fa);

      sUI num = fa->num_elements / 2;

      if(num > 0)
      {
         input_samples = fa->elements;
         num_samples   = num;
         input_offset  = 0;
         // while(input_offset >= num_samples)
         // {
         //    input_offset -= num_samples;
         // }

         // Ok
         return;
      }
   }
   // Nothing to play
   input_samples = NULL;
   input_offset  = 0;
   num_samples   = 0;
}

void SyModuleExtSample::setName (YAC_Object *_name) {
   ////printf("xxx SyModuleExtSample::setName\n");
   SyModule::setName(_name);

   ////printf("xxx SyModuleExtSample::setName: ui_audio_generator=%p\n", ui_audio_generator);

   // Synchronize with audio generator
   if(NULL != ui_audio_generator)
   {
      sUI nameKey = ui_audio_generator->yacMetaClassMemberGetAccessKeyByName("name");
      ////printf("xxx SyModuleExtSample::setName: nameKey=%u\n", nameKey);
      YAC_String *s = YAC_New_String();
      s->yacCopy(_name);
      YAC_Value v;
      v.initString(s, 1);
      ui_audio_generator->yacMetaClassMemberSet(nameKey, &v);
   }
}

YAC_Object *SyModuleExtSample::getName (void) {

   // Read from audio generator "name" metaclass property, if available
   if(NULL != ui_audio_generator)
   {
      sUI nameKey = ui_audio_generator->yacMetaClassMemberGetAccessKeyByName("name");
      YAC_Value v;
      v.initVoid();
      ui_audio_generator->yacMetaClassMemberGet(nameKey, &v);
      if(v.type >= YAC_TYPE_OBJECT)
      {
         return v.value.object_val;
      }
   }
   return SyModule::getName();
}

void SyModuleExtSample::uiSetAudioGenerator(YAC_Object *_audioGen) {
   ////printf("xxx SyModuleExtSample::uiSetAudioGenerator: audioGen=%p\n", _audioGen);
   ui_audio_generator = _audioGen;
}

YAC_Object *SyModuleExtSample::uiGetAudioGenerator(void) {
   return ui_audio_generator;
}

void SyModuleExtSample::uiSetAudioGeneratorId(sUI _audioGenId) {
   ////printf("xxx SyModuleExtSample::uiSetAudioGeneratorId: audioGenId=%u\n", _audioGenId);
   ui_audio_generator_id = _audioGenId;
}

sUI SyModuleExtSample::uiGetAudioGeneratorId(void) {
   return ui_audio_generator_id;
}

sBool SyModuleExtSample::saveStream(YAC_Object *_ofs) {
   // Write version
   _ofs->yacStreamWriteI16(0x0001);

   // Write ui_audio_generator_id
   _ofs->yacStreamWriteI16((sS16) ui_audio_generator_id);

   return 1;
}

sBool SyModuleExtSample::readStream(YAC_Object *_ifs) {
   // Read version
   sU16 v =_ifs->yacStreamReadI16();
   if(v >= 0x0001)
   {
      // Read ui_audio_generator_id
      ui_audio_generator_id = (sUI) (_ifs->yacStreamReadI16() & 65535);

      return 1;
   }

   return 0;
}
