// ----
// ---- file   : SyModuleExtSample.h
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 13Jun2010, 07Oct2010
// ----          
// ----
// ----
// ----

#ifndef __TKSYNTH_MODULE_EXT_SAMPLE_H__
#define __TKSYNTH_MODULE_EXT_SAMPLE_H__

YG("tksynth");


YC class SyModuleExtSample : public SyModule {
   // (note) This class is used as an interface to "external" sound generators
   //        

   sF32 *input_samples; // (reference to) l/r sample pairs
   sUI   num_samples;
   sUI   input_offset;  // increases by 1 each tick (0..num_samples-1)

   YAC_Object *ui_audio_generator;    // (opaque) reference to associated STX_AudioGenerator object
   sUI         ui_audio_generator_id; // set before saving an audio FX patch. this stores the index into the ST_Song::generators array

  public:
   YAC_POOLED(SyModuleExtSample, YAC_POOL_PRIORITY_LOW);

   void init   (void);
   void reset  (void);
   void doCalc (sUI _framesAdvance);

   // (note) the module name is synchronized with the ui_audio_generator::\"name\" metaclass property, if available.
   YM void        setName (YAC_Object *_name);
   YM YAC_Object *getName (void);

   // (note) the number of elements should be less or equal to the number of samples
   //        rendered in the SyModuleOutFloatArray::calcFloatArray() call.
   YM void setSamples(YAC_Object *_samples);

   YM void        uiSetAudioGenerator (YAC_Object *_audioGen);
   YM YAC_Object *uiGetAudioGenerator (void);

   YM void uiSetAudioGeneratorId (sUI _audioGenId);
   YM sUI  uiGetAudioGeneratorId (void);

   // overwrites SyModule::saveStream()
   sBool saveStream(YAC_Object *_ofs);

   // overwrites SyModule::readStream()
   sBool readStream(YAC_Object *_ifs);
};


#endif // __TKSYNTH_MODULE_EXT_SAMPLE_H__
