// ----
// ---- file   : SyModuleEQ3.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2090, 
// ----          22May2010, 30Sep2010, 07Oct2010
// ----
// ----
// ----

#include "tksynth.h"

#include "SyModuleFlt303.h"



//#define XXX_DEBUG defined

#ifdef XXX_DEBUG
static sSI xxx_smpcount = 0;
#define XXX_SMPCOUNT_IVAL ((1<<15)-1)
#endif // XXX_DEBUG


void SyModuleFlt303::init(void) {
   SyModule::init();

   allocInputs(6);

   addInput("smp",      SYPORT_MODE_SE, -1.0f, 1.0f,  0.0f); // #0
   addInput("freq",     SYPORT_MODE_BE,  0.0f, 1.0f,  1.0f); // #1
   addInput("q",        SYPORT_MODE_BE,  0.0f, 1.0f,  0.0f); // #2
   addInput("trigger",  SYPORT_MODE_BE, -1.0f, 1.0f, -1.0f); // #3
   addInput("envmod",   SYPORT_MODE_BE,  0.0f, 1.5f,  1.0f); // #4
   addInput("envdecay", SYPORT_MODE_BE,  0.0f, 1.0f,  0.1f); // #5


   allocOutputs(1);

   addOutput("smp", SYPORT_MODE_SE); // #0

}

void SyModuleFlt303::reset(void) {
   SyModule::reset();

   Dsyloopallch()
   {
      state[chIdx].reset();
   }

   c0           = 0;
   envpos       = 0;
   last_trigger = -1;
}

void SyModuleFlt303::doCalc(sUI _framesAdvance) {
   // (todo) filter decay is not mixrate independent (currently configured for 48KHz)

   /* compute result */
   SyInput *in_smp       = inputs[0];
   SyInput *in_freq      = inputs[1];
   SyInput *in_q         = inputs[2];
   SyInput *in_trigger   = inputs[3];
   SyInput *in_envmod    = inputs[4];
   SyInput *in_envdecay  = inputs[5];

   SyOutput *output = outputs[0];

   // Cutoff/center frequency
   sF32 freq = Dsyctl(in_freq);
   sF32 ra = ((rand() &65535)*(1.0f/32768.0f)) - 1.0f; // xxx analogue feel
   freq = freq * (1.0f + ra*0.2f);
   if(freq < 0.0f)
      freq = 0.0f;
   if(freq > 1.0f)
      freq = 1.0f;

   if(freq < 0.0005f)
      freq = 0.0005f;

   // Resonance
   sF32 q = Dsyctl(in_q);
   ra = ((rand() &65535)*(1.0f/32768.0f)) - 1.0f; // xxx analogue feel
   q = q + (ra*0.3f);

   if(q < 0)
      q = 0;
   else if(q > 1)
      q = 1;

   // Calc filter
   //filt_cutoff = 2.8 * freq;
   filt_cutoff = 1.6 * freq - 0.2;//1.0;//(3.0 * freq) -1.0;
   filt_reso   = q*1.1;
   env_mod     = Dsyctl(in_envmod);
   env_decay   = Dsyctl(in_envdecay);

   recalc_resonance();
   recalc_filter();
   recalc_decay();
   recalc_a_b();

  
   // check trigger
   if( 
      // ((last_trigger < 0) && (Dsyctl(in_trigger) > 0)) ||
      // ((last_trigger > 0) && (Dsyctl(in_trigger) < 0)) 
      ((last_trigger <= 0.0f) && (Dsyctl(in_trigger) > 0.0f))
      ////( (Dsyctl(in_trigger) > 0.0f) )
      )
   {
      /* Reset filter delta freq. */
      c0 = e1;
      envpos = 0;
      //printf("trig last_trigger=%f cur=%f\n", last_trigger, in_trigger->current_value[0]);
   }
   
   sF64 c = (1.0 - a - b) * 0.2;
   // // v = a * d1 + b * d2 + c * in_smp->current_value;
   // // sF64 out = 3.0 * v;
   // // output->current_value =  (sF32) (out * output->level + output->offset);


   Dsyloopch(in_smp)
   {
      Dsystate(SyModuleFlt303State);
      sF64 ab = a * st->d1 + b * st->d2;
      st->v = ab + c * Dsyval(in_smp);
      Dsyout(output, 3.0 * st->v);
   }


   // tick
   Dsyloopch(in_smp)
   {
      Dsystate(SyModuleFlt303State);
      st->shuffle();
   }

   if(++envpos >= envbound)
   {
      envpos = 0;
      c0 *= decay;
      recalc_a_b ();
   }


   last_trigger = Dsyctl(in_trigger);
}
