// ----
// ---- file   : SyModuleFlt303.h
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2011 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2090, 
// ----          22May2010, 07Oct2010, 23Jan2011
// ----
// ----
// ----


/* based on:
 *
 * DavBassFilter - DAV Bass Filter
 * Copyright (c) 1999, 2000 David A. Bartold, 2003 Tim Janik
 *
 * This is a TB-303 filter clone based on the VCF303 portions of
 * gsyn v0.2.  Code in update_locals() is copyright (c) 1998 Andy Sloane.
 *
 * Filter algorithm in function recalc_a_b() is based on the one
 * described in Musical Applications of Microprocessors by Hal Chamberlin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __TKSYNTH_MODULE_FLT_303_H__
#define __TKSYNTH_MODULE_FLT_303_H__

YG("tksynth");


class SyModuleFlt303State {
   // per-channel state
  public:
   sF64 d1, d2;
   sF64 v;

   inline void reset(void) {
      v = d1 = d2 = 0;
   }

   inline void shuffle(void) {
      d2 = d1;
      d1 = v;
   }
};

YC class SyModuleFlt303 : public SyModule {
   sF64 filt_cutoff, filt_reso, env_mod, env_decay;
   sF64 decay, resonance;
   sF64 a, b, c0;
   sF64 e0, e1;
   sSI envbound; /* 64 at 44100 */
   sSI envpos;
   sF32 last_trigger;
   SyModuleFlt303State state[SYPORT_MAXCHANNELS];

   inline void recalc_resonance ()
      {
         /* Update resonance. */
         resonance = exp (-1.20 + 3.455 * filt_reso);
      }
   
   inline void recalc_filter ()
      {
         /* Update vars given envmod, cutoff, and reso. */
         e0 = exp (5.613 - 0.8 * env_mod + 2.1553 * filt_cutoff - 0.7696 * (1.0 - filt_reso));
         e1 = exp (6.109 + 1.5876 * env_mod + 2.1553 * filt_cutoff - 1.2 * (1.0 - filt_reso));
         e0 *= sM_PI / mix_rate;
         e1 *= sM_PI / mix_rate;
         e1 -= e0;
      }
   
   inline void recalc_decay ()
      {
         /* Update decay given envdecay. */
         envbound = (sSI) (0.001452 * mix_rate); // 64 at 44100;
         envbound = sMAX (envbound, 1);
         double d = env_decay;
         d = 0.2 + (2.3 * d);
         d *= mix_rate;
         decay = pow (0.1, envbound / d);
      }
   
   inline void recalc_a_b ()
    {
      double whopping = e0 + c0;
      double k = exp (-whopping / resonance);
      a = 2.0 * cos (2.0 * whopping) * k;
      b = -k * k;
    }


  public:
   YAC_POOLED(SyModuleFlt303, YAC_POOL_PRIORITY_LOW);

   void init   (void);
   void reset  (void);
   void doCalc (sUI _framesAdvance);
};


#endif // __TKSYNTH_MODULE_FLT_303_H__

