// ----
// ---- file   : SyModuleFltBiquadRBJ.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2011 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2010, 
// ----          22May2010, 30Sep2010, 07Oct2010, 23Jan2011
// ----
// ----
// ----

#include "tksynth.h"

#include "SyModuleFltBiquadRBJ.h"


//#define XXX_DEBUG defined

#ifdef XXX_DEBUG
static sSI xxx_smpcount = 0;
#define XXX_SMPCOUNT_IVAL ((1<<15)-1)
#endif // XXX_DEBUG


void SyModuleFltBiquadRBJ::init(void) {
   SyModule::init();

   allocInputs(5);

   addInput("smp",    SYPORT_MODE_SE,     -1.0f,      1.0f,      0.0f); // #0
   addInput("type",   SYPORT_MODE_BE, (sF32)LPF, (sF32)HSH, (sF32)LPF); // #1
   addInput("dbgain", SYPORT_MODE_BE,      0.0f,    100.0f,      2.0f); // #2
   addInput("freq",   SYPORT_MODE_BE,      0.0f,      1.0f,      1.0f); // #3
   addInput("q",      SYPORT_MODE_BE,      0.0f,      1.0f,      0.0f); // #4


   allocOutputs(1);

   addOutput("smp", SYPORT_MODE_SE); // #0

   memset(state, 0, sizeof(SyModuleFltBiquadRBJState)*2);

}

void SyModuleFltBiquadRBJ::reset(void) {
   SyModule::reset();

   /* zero initial samples */
   Dsyloopallch()
   {
      Dsystate(SyModuleFltBiquadRBJState);
      st->reset();
   }
}

void SyModuleFltBiquadRBJ::calcQ(sF32 &freq, sF32 &bandwidth, sF32 &qScale, sF32 q) {
   bandwidth = q;

   sF32 ra = ((rand() &65535)*(1.0f/32768.0f)) - 1.0f; // xxx analogue feel
   bandwidth = bandwidth + (ra*0.05f);

   if(bandwidth < 0)
      bandwidth = 0;
   else if(bandwidth > 1)
      bandwidth = 1;

   qScale = 1.0f;

// #define Q_THRESHOLD 0.95f
// #define Q_FREQ_THRESHOLD 0.05f
//    if( (bandwidth >= Q_THRESHOLD) && (freq <= Q_FREQ_THRESHOLD) )
//    {
//       sF32 t = (bandwidth - Q_THRESHOLD) * (1.0f/(1.0f - Q_THRESHOLD));
//       t *= (Q_FREQ_THRESHOLD - freq) * (1.0f/Q_FREQ_THRESHOLD);
//       t = (t*t * (3.0f - 2.0f*t));
//       qScale *= 1.0f - t;
//       qScale *= qScale;
//       qScale *= qScale;
//       //printf("qScale=%f\n");
//    }
// //    else
// #define Q_FREQ_THRESHOLD2 0.005f
//    if(freq <= Q_FREQ_THRESHOLD2)
//    {
//       sF32 t = (Q_FREQ_THRESHOLD2 - freq) * (1.0f/Q_FREQ_THRESHOLD2);
//       t = (t*t * (3.0f - 2.0f*t));
//       qScale = 1.0f - t;
//       qScale *= qScale;
//       qScale *= qScale;
//    }

// #define Q_THRESHOLD2 0.8f
//    if(bandwidth > Q_THRESHOLD2)
//    {
//       sF32 t = (bandwidth - Q_THRESHOLD2) / (1.0f - Q_THRESHOLD2);
//       t = (t*t * (3.0f - 2.0f*t));
//       qScale *= (1.0f - 0.5f*t);
//    }

   bandwidth = 4.0f * (1.0f - bandwidth) + 0.3f;
}

void SyModuleFltBiquadRBJ::doCalc(sUI _framesAdvance) {

   /* compute result */
   SyInput *in_smp       = inputs[0];
   SyInput *in_type      = inputs[1];
   SyInput *in_dbgain    = inputs[2];
   SyInput *in_freq      = inputs[3];
   SyInput *in_q         = inputs[4];

   SyOutput *output = outputs[0];
   
   // Filter type
   sSI type = (sSI) Dsyctl(in_type);
   if(type < 0)
      type = 0;
   else if(type > HSH)
      type = HSH;


   // Cutoff/center frequency
   sF32 freq = Dsyctl(in_freq);
   //printf("xxx freq=%f\n", freq);

   sF32 ra = ((rand()&65535) * (1.0f/32768.0f)) - 1.0f; // xxx analogue feel
   freq = freq * (1.0f + ra*0.15f);


   if(freq < 0.0f)
      freq = 0.0f;
   if(freq > 1.0f)
      freq = 1.0f;

   if(freq < 0.0005f)
      freq = 0.0005f;


   // Resonance
   sF32 bandwidth, qScale;
   calcQ(freq, bandwidth, qScale, Dsyctl(in_q));


// #ifdef XXX_DEBUG
//    if(((xxx_smpcount)&XXX_SMPCOUNT_IVAL)==0)
//    {
//       printf("-----------------------------------------------\n");
//       printf("     type=%d\n", type);
//       printf("   dbgain=%f\n", in_dbgain->current_value);
//       printf("     freq=%f\n", freq);
//       printf("bandwidth=%f\n", bandwidth);
//       printf("    insmp=%f\n", in_smp->current_value);
//    }
// #endif // XXX_DEBUG

   // Recalc coefficients
   calcCoeff(type, 
             Dsyctl(in_dbgain), 
             freq * 0.5f * 0.5f * (sF32)mix_rate, (sF32)mix_rate,
             bandwidth
             );

   // Dyac_dbgflt_32(freq);
   // Dyac_dbgflt_32(bandwidth);


   Dsyloopch(in_smp)
   {
      Dsystate(SyModuleFltBiquadRBJState);

      sF32 inSmp = Dsyval(in_smp);

      ////printf("xxx inSmp=%f\n", inSmp);

      inSmp = Dyac_denorm_32(inSmp);

      // // Dyac_dbgflt_32(inSmp);
      // // Dyac_dbgflt_32(qScale);
      // // Dyac_dbgflt_32(biquad.a0);
      // // Dyac_dbgflt_32(biquad.a1);
      // // Dyac_dbgflt_32(biquad.a2);
      // // Dyac_dbgflt_32(biquad.a3);
      // // Dyac_dbgflt_32(biquad.a4);
      // // Dyac_dbgflt_32(st->x1);
      // // Dyac_dbgflt_32(st->x2);
      // // Dyac_dbgflt_32(st->y1);
      // // Dyac_dbgflt_32(st->y2);
      
      
      sF32 o = (sF32) (
         qScale * biquad.a0 * qScale * inSmp + 
         biquad.a1 * st->x1 + 
         biquad.a2 * st->x2 -
         biquad.a3 * st->y1 - 
         biquad.a4 * st->y2 );

      if(o < -10.0f) // (todo) is this check really necessary ?
         o = 0.0f;
      else if(o > 10.0f)
         o = 0.0f;

      o = Dyac_denorm_32(o);
      
      Dsyout(output, o);
   }

// #ifdef XXX_DEBUG
//    if(((xxx_smpcount)&XXX_SMPCOUNT_IVAL)==0)
//    {
//       printf("biquad: a0=%f a1=%f a2=%f a3=%f\n", biquad.a0, biquad.a1, biquad.a2, biquad.a3);
//       printf("biquad: x1=%f x2=%f y1=%f y2=%f\n", biquad.x1, biquad.x2, biquad.y1, biquad.y2);
//       printf("   outsmp=%f\n", output->current_value);
//    }
//    xxx_smpcount++;
// #endif // XXX_DEBUG


   // tick

   // Shuffle history
   Dsyloopch(in_smp)
   {
      Dsystate(SyModuleFltBiquadRBJState);

      /* shift x1 to x2, sample to x1 */
      st->x2 = st->x1;
      st->x1 = Dsyval(in_smp) /** qScale*/;
      st->x1 = Dyac_denorm_32(st->x1);
      
      /* shift y1 to y2, result to y1 */
      st->y2 = st->y1;
      st->y1 = Dsyval(output);
   }

}

void SyModuleFltBiquadRBJ::calcCoeff(sSI  type,
                                     sF32 dbGain,   // gain of filter
                                     sF32 freq,     // center frequency (0..1)
                                     sF32 srate,    // sampling rate
                                     sF32 bandwidth // bandwidth in octaves
                                     )
{
   sF32 A, omega, sn, cs, alpha, beta;
   sF32 a0, a1, a2, b0, b1, b2;

   /* setup variables */
   A = powf(10, dbGain /40);
   omega = (sF32) (2 * sM_PI * freq /srate);
   sn = sin(omega);
   cs = cos(omega);
   alpha = (sF32) (sn * sinh(sM_LN2 /2 * bandwidth * omega /sn));
   beta = sqrt(A + A);

   switch (type)
   {
      default:
      case LPF:
         b0 = (1 - cs) /2;
         b1 = 1 - cs;
         b2 = (1 - cs) /2;
         a0 = 1 + alpha;
         a1 = -2 * cs;
         a2 = 1 - alpha;
         break;

      case HPF:
         b0 = (1 + cs) /2;
         b1 = -(1 + cs);
         b2 = (1 + cs) /2;
         a0 = 1 + alpha;
         a1 = -2 * cs;
         a2 = 1 - alpha;
         break;

      case BPF:
         b0 = alpha;
         b1 = 0;
         b2 = -alpha;
         a0 = 1 + alpha;
         a1 = -2 * cs;
         a2 = 1 - alpha;
         break;

      case NOTCH:
         b0 = 1;
         b1 = -2 * cs;
         b2 = 1;
         a0 = 1 + alpha;
         a1 = -2 * cs;
         a2 = 1 - alpha;
         break;
         
      case PEQ:
         b0 = 1 + (alpha * A);
         b1 = -2 * cs;
         b2 = 1 - (alpha * A);
         a0 = 1 + (alpha /A);
         a1 = -2 * cs;
         a2 = 1 - (alpha /A);
         break;
         
      case LSH:
         b0 = A * ((A + 1) - (A - 1) * cs + beta * sn);
         b1 = 2 * A * ((A - 1) - (A + 1) * cs);
         b2 = A * ((A + 1) - (A - 1) * cs - beta * sn);
         a0 = (A + 1) + (A - 1) * cs + beta * sn;
         a1 = -2 * ((A - 1) + (A + 1) * cs);
         a2 = (A + 1) + (A - 1) * cs - beta * sn;
         break;
         
      case HSH:
         b0 = A * ((A + 1) + (A - 1) * cs + beta * sn);
         b1 = -2 * A * ((A - 1) + (A + 1) * cs);
         b2 = A * ((A + 1) + (A - 1) * cs - beta * sn);
         a0 = (A + 1) - (A - 1) * cs + beta * sn;
         a1 = 2 * ((A - 1) - (A + 1) * cs);
         a2 = (A + 1) - (A - 1) * cs - beta * sn;
         break;
   }
   
   /* precompute the coefficients */
   biquad.a0 = b0 /a0;
   biquad.a1 = b1 /a0;
   biquad.a2 = b2 /a0;
   biquad.a3 = a1 /a0;
   biquad.a4 = a2 /a0;
}
