// ----
// ---- file   : SyModuleFltBiquadRBJ.h
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2090, 
// ----          22May2010, 07Oct2010
// ----
// ----
// ----

#ifndef __TKSYNTH_MODULE_FLT_BIQUAD_RBJ_H__
#define __TKSYNTH_MODULE_FLT_BIQUAD_RBJ_H__

YG("tksynth");

//
// based on Robert Bristow-Johnson's Cookbook formulae for audio EQ biquad filter coefficients
//  <http://musicdsp.org/files/Audio-EQ-Cookbook.txt>
//
// and Tom St Denis' "biquad.c" public domain implemention 
//  http://musicdsp.org/files/biquad.c, <http://tomstdenis.home.dhs.org>
//

class SyModuleFltBiquadRBJState {
   // per channel state
  public:
   sF32 x1, x2, y1, y2;

   inline void reset(void) {
      x1 = x2 = 0;
      y1 = y2 = 0;
   }
};

YC class SyModuleFltBiquadRBJ : public SyModule {
   struct {
      sF32 a0, a1, a2, a3, a4;
   } biquad;

   SyModuleFltBiquadRBJState state[SYPORT_MAXCHANNELS];

  public:
   YAC_POOLED(SyModuleFltBiquadRBJ, YAC_POOL_PRIORITY_LOW);

   /* filter types */
   enum {
      LPF, /* low pass filter */
      HPF, /* High pass filter */
      BPF, /* band pass filter */
      NOTCH, /* Notch Filter */
      PEQ, /* Peaking band EQ filter */
      LSH, /* Low shelf filter */
      HSH /* High shelf filter */
   };


   void calcCoeff(sSI type, 
                  sF32 dbGain,   // gain of filter
                  sF32 freq,     // center frequency
                  sF32 srate,    // sampling rate
                  sF32 bandwidth // bandwidth in octaves
                  );

   void calcQ(sF32 &freq, sF32 &bandwidth, sF32 &qScale, sF32 q);

   void init   (void);
   void reset  (void);
   void doCalc (sUI _framesAdvance);
};


#endif // __TKSYNTH_MODULE_FLT_BIQUAD_RBJ_H__

