// ----
// ---- file   : SyModuleFltMoog1.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2011 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2090, 
// ----          22May2010, 25Sep2010, 30Sep2010, 07Oct2010, 23Jan2011
// ----
// ----
// ----

#include "tksynth.h"

#include "SyModuleFltMoog1.h"

// see <http://musicdsp.org/showArchiveComment.php?ArchiveID=24>


//#define XXX_DEBUG defined

#ifdef XXX_DEBUG
static sSI xxx_smpcount = 0;
#define XXX_SMPCOUNT_IVAL ((1<<15)-1)
#endif // XXX_DEBUG


void SyModuleFltMoog1::init(void) {
   SyModule::init();

   allocInputs(4);

   addInput("smp",  SYPORT_MODE_SE,     -1.0f,      1.0f,      0.0f); // #0
   addInput("type", SYPORT_MODE_BE, (sF32)LPF, (sF32)BPF, (sF32)LPF); // #1
   addInput("freq", SYPORT_MODE_BE,      0.0f,      1.0f,      1.0f); // #2
   addInput("q",    SYPORT_MODE_BE,      0.0f,      1.0f,      0.0f); // #3


   allocOutputs(1);

   addOutput("smp", SYPORT_MODE_SE); // #0
}

void SyModuleFltMoog1::reset(void) {
   SyModule::reset();

   param.p = param.k = param.r = 0.0f;

   Dsyloopallch()
   {
      Dsystate(SyModuleFltMoog1State);

      st->reset();
   }
}


void SyModuleFltMoog1::calcCoeff(sSI  type,
                                 sF32 freq,
                                 sF32 srate,
                                 sF32 q
                                 )
{
   param.p = freq * (1.8f - 0.8f * freq);
   param.k = param.p + param.p - 1.0f;

   sF32 t  = (1.0f - param.p) * 1.386249f;
   sF32 t2 = 12.0f + t * t;
   param.r = q * (t2 + 6.0f * t) / (t2 - 6.0f * t);

}

void SyModuleFltMoog1::doCalc(sUI _framesAdvance) {

   /* compute result */
   SyInput *in_smp       = inputs[0];
   SyInput *in_type      = inputs[1];
   SyInput *in_freq      = inputs[2];
   SyInput *in_q         = inputs[3];

   SyOutput *output = outputs[0];
   
   // Filter type
   sSI type = (sSI) Dsyctl(in_type);
   if(type < 0)
      type = 0;
   else if(type > BPF)
      type = BPF;

   // Cutoff/center frequency
   sF32 freq = Dsyctl(in_freq);
   sF32 ra = ((rand() &65535)*(1.0f/32768.0f)) - 1.0f; // xxx analogue feel
   freq = freq * (1.0f + ra*0.2f);
   if(freq < 0.0f)
      freq = 0.0f;
   if(freq > 1.0f)
      freq = 1.0f;

   if(freq < 0.0005f)
      freq = 0.0005f;


   // Resonance
   sF32 q = Dsyctl(in_q);
   ra = ((rand() &65535)*(1.0f/32768.0f)) - 1.0f; // xxx analogue feel
   q = q + (ra*0.01f);

   // Dyac_dbgflt_32(q);

   if(q < 0.0f)
      q = 0.0f;
   else if(q > 1.0f)
      q = 1.0f;


// #ifdef XXX_DEBUG
//    if(((xxx_smpcount)&XXX_SMPCOUNT_IVAL)==0)
//    {
//       printf("-----------------------------------------------\n");
//       printf("     type=%d\n", type);
//       printf("     freq=%f\n", freq);
//       printf("        q=%f\n", q);
//       printf("    insmp=%f\n", in_smp->current_value);
//    }
// #endif // XXX_DEBUG

   // Recalc coefficients
   calcCoeff(type, 
             freq, 
             (sF32)mix_rate, 
             q
             );

#define FREQ_THRESHOLD 0.4
   sF32 t = 1.0f;
   if(freq <= FREQ_THRESHOLD)
   {
      sF32 f = (sF32) (freq / FREQ_THRESHOLD);
      t = q * (1.0f-f);
      t = t*t * (3.02f - 2.0f*t);
      t = (1.0f + t * 1.13f);
   }

   // Iterate channels
   Dsyloopch(in_smp)
   {
      Dsystate(SyModuleFltMoog1State);

      sF32 inSmp = Dsyval(in_smp);

      inSmp = Dyac_denorm_32(inSmp);

      // Dyac_dbgflt_32(inSmp);
      // Dyac_dbgflt_32(param.p);
      // Dyac_dbgflt_32(param.k);
      // Dyac_dbgflt_32(param.r);
      // Dyac_dbgflt_32(st->x);
      // Dyac_dbgflt_32(st->oldx);
      // Dyac_dbgflt_32(st->y1);
      // Dyac_dbgflt_32(st->y2);
      // Dyac_dbgflt_32(st->y3);
      // Dyac_dbgflt_32(st->y4);
      // Dyac_dbgflt_32(st->oldy1);
      // Dyac_dbgflt_32(st->oldy2);
      // Dyac_dbgflt_32(st->oldy3);

      // Inverted feed back for corner peaking
      st->x = inSmp - param.r * st->y4;
      st->x = Dyac_denorm_32(st->x);

      // // sF32 xxxOldY1 = st->y1;

      //Four cascaded onepole filters (bilinear transform)
      st->y1    = st->x *param.p + st->oldx *param.p - st->y1*param.k;
      // // if(Dyac_chkdenorm_32(st->y1))
      // // {
      // //    printf("xxx denorm y1, xxxOldY1=%f y1=%f  oldy1=0x%08x y1=0x%08x\n", xxxOldY1, st->y1, *(sUI*)&xxxOldY1, *(sUI*)&st->y1);
      // //    // 
      // //    // 33222222222211111111110000000000
      // //    // 10987654321098765432109876543210
      // //    //          10000011000100010110001 = 0x004188B1
      // //    // 10000000110101100100110001000010
      // // }
      st->y2    = st->y1*param.p + st->oldy1*param.p - st->y2*param.k;
      st->y3    = st->y2*param.p + st->oldy2*param.p - st->y3*param.k;
      st->newy4 = st->y3*param.p + st->oldy3*param.p - st->y4*param.k;   

      // Dyac_dbgflt_32(st->newy4);

      st->y1    = Dyac_denorm_32(st->y1);
      st->y2    = Dyac_denorm_32(st->y2);
      st->y3    = Dyac_denorm_32(st->y3);
      st->newy4 = Dyac_denorm_32(st->newy4);

      //Clipper band limited sigmoid
      st->newy4 = st->newy4 - ((st->newy4*st->newy4*st->newy4)/6.0f);
      // // st->newy4 = st->newy4;

      sF32 out = t * st->newy4;

      if(HPF == type)
      {
         out = inSmp - out;
      }
      else if(BPF == type)
      {
         out = out - (inSmp - out);
      }

      // Dyac_dbgflt_32(out);

      out = Dyac_denorm_32(out);
      
      Dsyout(output, out);
   }


   Dsyloopch(in_smp)
   {
      Dsystate(SyModuleFltMoog1State);
      st->shuffle();
   }

}

