// ----
// ---- file   : SyModuleFltMoog1.h
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2090, 
// ----          22May2010, 07Oct2010
// ----
// ----
// ----

#ifndef __TKSYNTH_MODULE_FLT_MOOG_1_H__
#define __TKSYNTH_MODULE_FLT_MOOG_1_H__

YG("tksynth");


class SyModuleFltMoog1State {
  public:
   sF32 x, y1, y2, y3, y4; // temporary
   sF32 oldx, oldy1, oldy2, oldy3;
   sF32 newy4;

   inline void reset(void) {
      x = y1 = y2 = y3 = y4 = 0.0f;
      oldx = oldy1 = oldy2 = oldy3 = 0.0f;
      newy4 = 0.0f;
   }

   inline void shuffle(void) {
      oldx  = x;
      oldy1 = y1;
      oldy2 = y2;
      oldy3 = y3;
      y4    = newy4;
   }
};

YC class SyModuleFltMoog1 : public SyModule {
   struct {
      sF32 p, k, r;
   } param;

   SyModuleFltMoog1State state[2];

  public:
   YAC_POOLED(SyModuleFltMoog1, YAC_POOL_PRIORITY_LOW);

   /* filter types */
   enum {
      LPF, /* low pass filter */
      HPF, /* High pass filter */
      BPF, /* band pass filter */
   };


   void calcCoeff(sSI type, 
                  sF32 freq,   // center frequency 0..1
                  sF32 srate,  // sampling rate
                  sF32 q       // resonance 0..1
                  );

   void init   (void);
   void reset  (void);
   void doCalc (sUI _framesAdvance);
};


#endif // __TKSYNTH_MODULE_FLT_MOOG_1_H__
