// ----
// ---- file   : SyModuleIO.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2090, 
// ----          22May2010, 30Sep2010, 11Oct2010
// ----
// ----
// ----

#define YAC_EPSILONCOMPARE_ABS defined
#define YAC_EPSILONCOMPARE_ABS_DEFAULT defined

#include "tksynth.h"

#include "SyModuleIO.h"


void SyModuleIO::init(void) {
   SyModule::init();

   sUI i;

   allocInputs(SYMODULEIO_MAX_IO_CHANNELS);
   allocOutputs(SYMODULEIO_MAX_IO_CHANNELS);
   for(i=0; i<SYMODULEIO_MAX_IO_CHANNELS; i++)
   {
      char buf[32];
      sprintf(buf, "#%d", (i+1));
      addInput(buf, SYPORT_MODE_BE, 0.0f, 1.0f, 0.0f);
      addOutput(buf, SYPORT_MODE_BE);
   }

   if(0)
   {
      // if used to connect sample streams between structures

      // Configure output 0 ("smp") to stereo
      {
         SyOutput *outp = (SyOutput *) getOutputById(0); // "smp"
         outp->setNumChannels(2);
      }

      // Configure input 0 ("smp") to accept values in range -1..1
      {
         SyInput *inp = (SyInput *) getInputById(0); // "smp"
         inp->setNumChannels(2);
         inp->setMinResetValue(-1.0f);
         inp->setMaxResetValue( 1.0f);
      }
   }
   else
   {
      // If used to input external parameters to a structure

      // Configure output 0 to mono
      {
         SyOutput *outp = (SyOutput *) getOutputById(0);
         outp->setNumChannels(1);
      }

      // Configure input 0 to accept values in range 0..1
      {
         SyInput *inp = (SyInput *) getInputById(0);
         inp->setNumChannels(1);
         inp->setMinResetValue(0.0f);
         inp->setMaxResetValue(1.0f);
      }
   }

   num_used_inputs  = 1;
   num_used_outputs = 1;
}

void SyModuleIO::reset(void) {
   SyModule::reset();
}

void SyModuleIO::doCalc(sUI _framesAdvance) {

   ////printf("xxx SyModuleIO::doCalc: this=%p\n", this);

   sUI i;
   // Copy input to ouput values
   /////for(i=0; i<SYMODULEIO_MAX_IO_CHANNELS; i++)
   for(i=0; i<num_used_inputs; i++)
   {
      SyInput *input = inputs[i];
      SyOutput *output = outputs[i];
      Dsyloopch(input)
      {
         Dsyout(output, Dsyval(input));
         ////output->current_value = (sF32) (input->current_value * output->level + output->offset);
         //printf("xxx SyModuleIO: inIdx=%d\n", i);
      }

      if(SYINPUT_CLAMP_TRIG == input->clamp_mode)
      {
         // Reset trigger-style input back to 0 after outputting the current value
         ////printf("xxx SyModuleIO: CLAMP_TRIG reset to 0 (oldval=%f curval=%f)\n", input->reset_value, input->current_value[0]);
         input->reset_value = 0.0f;
      }

   }
}

void SyModuleIO::doCalcBlock(sUI _numFrames) {

   // (todo) add support for SE mode I/O

   sUI i;
   for(i=0; i<num_used_inputs; i++)
   {
      SyInput *input = inputs[i];
      SyOutput *output = outputs[i];

      sUI numSamples = _numFrames * input->num_channels;

      ////printf("xxx input %u/%u numOut=%u mode=%u\n", i, num_used_inputs, num_used_outputs, input->mode);

      if(Dfltequal(output->level, 1.0f) && Dfltequal(output->offset, 0.0f))
      {
         // Copy as-is
         memcpy(output->buffer, input->buffer, numSamples * sizeof(sF32));
      }
      else
      {
         // Apply offset/scaling
         sUI k;
         for(k=0; k<numSamples; k++)
         {
            output->buffer[k] = input->buffer[k] * output->level + output->offset;
            //printf("out[%u] = %f\n", k, output->buffer[k]);
         }
      }
   }
}

sUI SyModuleIO::getNumInputs(void) {
   return num_used_inputs;
}

sUI SyModuleIO::getNumOutputs(void) {
   return num_used_outputs;
}

void SyModuleIO::setNumInputs(sUI _numIn) {
   if(_numIn < num_inputs)
   {
      if(0 == _numIn)
      {
         _numIn = 1;
      }
      num_used_inputs = _numIn;
   }
   else
   {
      num_used_inputs = num_inputs;
   }
}

void SyModuleIO::setNumOutputs(sUI _numOut) {
   if(_numOut < num_outputs)
   {
      if(0 == _numOut)
      {
         _numOut = 1;
      }
      num_used_outputs = _numOut;
   }
   else
   {
      num_used_outputs = num_outputs;
   }
}
