// ----
// ---- file   : SyModuleOpMulAsym.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2013 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 29May2013
// ----
// ----
// ----

#include "tksynth.h"

#include "SyModuleOpMulAsym.h"


void SyModuleOpMulAsym::init(void) {
   SyModule::init();

   allocInputs(3);
   addInput("smp",     SYPORT_MODE_SE, -1.0f,  1.0f, 0.0f);  // #0
   addInput("mul_neg", SYPORT_MODE_BE, -1.0f,  1.0f, 0.0f);  // #1
   addInput("mul_pos", SYPORT_MODE_BE, -1.0f,  1.0f, 0.0f);  // #2

   allocOutputs(1);
   addOutput("smp", SYPORT_MODE_SE); // #0
}

void SyModuleOpMulAsym::reset(void) {
   SyModule::reset();
}

void SyModuleOpMulAsym::doCalc(sUI _framesAdvance) {

   SyInput *in_smp     = inputs[0];
   SyInput *in_mul_neg = inputs[1];
   SyInput *in_mul_pos = inputs[2];

   SyOutput *out_smp = outputs[0];

   sF32 mulNeg = Dsyctl(in_mul_neg);
   sF32 mulPos = Dsyctl(in_mul_pos);

   Dsyloopch(in_smp)
   {
      sF32 out;
      sF32 a = Dsyval(in_smp);

      if(a < 0)
      {
         out = a * mulNeg;
      }
      else
      {
         out = a * mulPos;
      }

      Dsyout(out_smp, out);
   }
}
