// ----
// ---- file   : SyModuleOsc.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2090, 
// ----          22May2010, 06Jun2010, 02Jul2010
// ----
// ----
// ----

#include "tksynth.h"

#include "SyModuleOsc.h"


SyModuleOsc::SyModuleOsc(void) {
   freq_table    = NULL;
   b_linear_freq = 0;
   b_synczerox   = 0;
}

SyModuleOsc::~SyModuleOsc() {
}

void SyModuleOsc::setFreqTable(YAC_Object *_freqTable) {
   if(YAC_BCHK(_freqTable, YAC_CLID_FLOATARRAY))
   {
      YAC_CAST_ARG(YAC_FloatArray, fa, _freqTable);

      if(fa->num_elements >= 128)
      {
         freq_table = fa->elements;
      }
      else
      {
         yac_host->printf("[---] SyModuleOsc::setFreqTable: array too small (%d < 128).\n", fa->num_elements);
      }
   }
   else
   {
      freq_table = NULL;
   }
}

void SyModuleOsc::setEnableLinear(sBool _bEnabled) {
   b_linear_freq = _bEnabled;
}

sBool SyModuleOsc::getEnableLinear(void) {
   return b_linear_freq;
}

void SyModuleOsc::setEnableSyncZeroX(sBool _bEnabled) {
   b_synczerox = _bEnabled;
}

sBool SyModuleOsc::getEnableSyncZeroX(void) {
   return b_synczerox;
}

sBool SyModuleOsc::saveStream(YAC_Object *_ofs) {
   return saveStreamOSC(_ofs);
}

sBool SyModuleOsc::saveStreamOSC(YAC_Object *_ifs) {
   // Write version
   _ifs->yacStreamWriteI16(0x0002);

   // Write b_linear
   _ifs->yacStreamWriteI8((sU8)b_linear_freq);

   // Write b_synczerox
   _ifs->yacStreamWriteI8((sU8)b_synczerox);

   return 1;
}

sBool SyModuleOsc::readStream(YAC_Object *_ifs) {
   return readStreamOSC(_ifs);
}

sBool SyModuleOsc::readStreamOSC(YAC_Object *_ifs) {
   // Read version
   sU16 v =_ifs->yacStreamReadI16();
   if(v >= 0x0001)
   {
      // Read b_linear
      b_linear_freq = (sBool) _ifs->yacStreamReadI8();
   }

   if(v >= 0x0002)
   {
      // Read b_synczerox
      b_synczerox = (sBool) _ifs->yacStreamReadI8();
   }

   return 1;
   ///return (v == 0x0000);
}
