// ----
// ---- file   : SyModuleOscSHI.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2010-2011 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 22Sep2010, 30Sep2010, 07Oct2010, 23Jan2011
// ----          
// ----
// ----
// ----

#include "tksynth.h"

#include "SyModuleOscSHI.h"


void SyModuleOscSHI::init(void) {
   SyModule::init();

   allocInputs(4);
   addInput("minamp", SYPORT_MODE_BE, -10000.0f     , 10000.0f,  -1.0f);  // #0
   addInput("maxamp", SYPORT_MODE_BE, -10000.0f     , 10000.0f,   1.0f);  // #1
   addInput("minspd", SYPORT_MODE_BE,      0.000001f, 10000.0f,   1.0f);  // #2 milliseconds
   addInput("maxspd", SYPORT_MODE_BE,      0.000001f, 10000.0f, 100.0f);  // #3 milliseconds

   allocOutputs(1);
   addOutput("smp", SYPORT_MODE_SE);  // #0
}

void SyModuleOscSHI::reset(void) {
   SyModule::reset();

   time = 1.0f;
   spd  = 1.0f;

   cval = 0.0f;
   nval = 0.0f;
}

void SyModuleOscSHI::doCalc(sUI _framesAdvance) {

   SyInput *in_minamp = inputs[0];
   SyInput *in_maxamp = inputs[1];
   SyInput *in_minspd = inputs[2];
   SyInput *in_maxspd = inputs[3];

   SyOutput *output   = outputs[0];

   time += spd;

   if(time >= 1.0f)
   {
      time = 0.0f;

      sF32 min = Dsyctl(in_minamp);
      sF32 max = Dsyctl(in_maxamp);

      if(min > max)
      {
         sF32 t = min;
         min = max;
         max = t;
      }

      cval = nval;
      if(cval < min) // clipping needed for first sample
      {
         cval = min;
      }
      else if(cval > max)
      {
         cval = max;
      }

      nval = (sF32) ((rand()*(0.999999999999f/((sF32)RAND_MAX))) * (max - min) ) + min;

      sF32 minSpd = Dsyctl(in_minspd);
      sF32 maxSpd = Dsyctl(in_maxspd);

      if(minSpd > maxSpd)
      {
         sF32 t = minSpd;
         minSpd = maxSpd;
         maxSpd = t;
      }

      spd = (sF32) ((rand()*(0.999999999999f/((sF32)RAND_MAX))) * (maxSpd - minSpd) ) + minSpd;

      sSI spdNumSamples = (sSI) (spd * mix_rate / 1000.0f);

      if(spdNumSamples <= 0)
         spdNumSamples = 1;

      spd = _framesAdvance * 1.0f / spdNumSamples;
   }

   sF32 t = time;
   {
      // Smoothstep:
      t = t * t * (3.0f - 2.0f * t);
   }
   sF32 out = (sF32) (cval + (nval - cval) * t);

   // printf("xxx out=%f cval=%f nval=%f t=%f spd=%f\n", out, cval, nval, time, spd);


   Dsyoutmono(output, out);
}
