// ----
// ---- file   : SyModuleOscSawDSFBlit.h
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2090, 
// ----          22May2010, 07Oct2010
// ----
// ----
// ----

#ifndef __TKSYNTH_MODULE_OSC_SAW_DSF_BLIT_H__
#define __TKSYNTH_MODULE_OSC_SAW_DSF_BLIT_H__

YG("tksynth");


YC class SyModuleOscSawDSFBLIT : public SyModuleOsc {

   struct {
      // dsf blit
      sF64 phase;    /* phase accumulator */
      sF64 aNQ;      /* attenuation at nyquist */
      sF64 curcps;   /* current frequency, updated once per cycle */
      sF64 curper;   /* current period, updated once per cycle */
      sF64 N;        /* # partials */
      sF64 a;        /* dsf parameter which controls roll-off */
      sF64 aN;       /* former to the N */

      /* Leaky integrator/first order lowpass which 
       * shapes the impulse train into a nice
       * -6dB/octave spectrum
       *
       * The cutoff frequency needs to be pretty lowish
       * or the sawtooth will suffer phase distortion
       * at the low end.
       */
      struct {
         sF64 x1, y1;
         sF64 a, b;
         sF64 newy; // temporary, used in both calc() and tick() methods
      } lowpass;

      sF64 val;  // temporary, used in both calc() and tick() methods
   } blit;


   void initLowpass   (sF64 cutoff);
   ////sF64 updateLowpass (sF64 x);
   void initBlit      (sF64 aNQ, sF64 cutoff);
   sF64 updateBlit    (sF64 cps, sF64 phase);

  public:
   YAC_POOLED(SyModuleOscSawDSFBLIT, YAC_POOL_PRIORITY_LOW);

   void init   (void);
   void reset  (void);
   void doCalc (sUI _framesAdvance);
};


#endif // __TKSYNTH_MODULE_OSC_SAW_DSF_BLIT_H__
