// ----
// ---- file   : SyModuleOscSinePure.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2013 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2090, 
// ----          23May2010, 02Jul2010, 30Sep2010, 07Oct2010, 11Oct2010, 23Jan2011, 30May2013
// ----
// ----
// ----

#include "tksynth.h"

#include "SyModuleOscSinePure.h"


void SyModuleOscSinePure::init(void) {
   SyModule::init();

   allocInputs(4);
   addInput("volume", SYPORT_MODE_BE, -4.0f,    4.0f,   1.0f);  // #0
   addInput("pitch",  SYPORT_MODE_BE,  5.0f, 2000.0f, 440.0f);  // #1
   addInput("phase",  SYPORT_MODE_BE,  0.0f,    1.0f,   0.0f);  // #2
   addInput("sync",   SYPORT_MODE_BE,  0.0f,    1.0f,   0.0f);  // #3

   allocOutputs(2);
   addOutput("smp",  SYPORT_MODE_SE);   // #0
   addOutput("sync", SYPORT_MODE_SE);   // #1

   // sync_lastval = 0.0f;
   // sync_interpol = 0.0f;
}

void SyModuleOscSinePure::reset(void) {
   SyModule::reset();

   angle = 0.0;
}

void SyModuleOscSinePure::doCalc(sUI _framesAdvance) {

   SyInput *in_volume = inputs[0];
   SyInput *in_pitch  = inputs[1];
   SyInput *in_phase  = inputs[2];
   SyInput *in_sync   = inputs[3];

   SyOutput *output   = outputs[0];
   SyOutput *out_sync = outputs[1];

   // Oscillator sync
   sF32 sync = Dsyctl(in_sync);
   if(b_synczerox)
   {
      if(Dsyfltsign(sync) ^ Dsyfltsign(Dsylastctl(in_sync)))
      {
         angle = 0.0f;
      }
   }
   else
   {
      if(sync > 0.0f)
      {
         angle = 0.0f;
      }
   }

   sF32 ph = Dsyctl(in_phase);

      // printf("ph=%f\n", ph);

   sF32 vol = Dsyctl(in_volume);

   // crash test:
   // if(vol > 50)
   //    ((sU8*)0)[0] = 0;
   
   sF32 out = (sF32) (sin(angle + ph * sM_PI * 2.0f) * vol);

   Dsyoutmono(output, out);

   sF64 w = (sM_PI*2) * (mapOscPitch(Dsyctl(in_pitch)) / mix_rate);

   angle += w * _framesAdvance;

   while(angle >= (sM_PI*2))
   {
      angle -= (sM_PI*2);
      sync = 1.0f;
   }
   while(angle < 0)
   {
      angle += (sM_PI*2);
      sync = 1.0f;
   }
   
   Dsyoutmono(out_sync, sync);
}

void SyModuleOscSinePure::doCalcBlock(sUI _numFrames) {

   // (todo) there are a number of features and therefore optimizations that could be
   //        made even if this is a really simple module:
   //         - volume ramping
   //         - pitch ramping
   //         - support for oscillator sync
   //         - support for sample-exact volume, pitch, and/or sync input

   SyInput *in_volume = inputs[0];
   SyInput *in_pitch  = inputs[1];
   SyInput *in_phase  = inputs[2];
   ////SyInput *in_sync   = inputs[3];

   if( (SYPORT_MODE_BE != in_phase->mode)  ||
       (SYPORT_MODE_BE != in_volume->mode) ||
       (SYPORT_MODE_BE != in_pitch->mode)
       )
   {
      SyModule::doCalcBlock(_numFrames);
      return;
   }

   SyOutput *output   = outputs[0];
 
   sF64 w = (sM_PI*2) * (mapOscPitch(Dsyctlbm(in_pitch)) / mix_rate);

   sF64 a = angle + Dsyctlbm(in_phase) * sM_PI * 2.0f;

   sF32 vol = Dsyctlbm(in_volume);

   for(sUI i=0; i<_numFrames; i++)
   {
      sF32 out = (sF32) (sin(a) * vol);

      output->buffer[i] = out;

      a += w;
   }

   a -= Dsyctlbm(in_phase);

   while(a >= (sM_PI*2))
   {
      a -= (sM_PI*2);
   }
   while(a < 0)
   {
      a += (sM_PI*2);
   }

   angle = a;
}
