// ----
// ---- file   : SyModuleOscTriPure.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2011 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2090, 
// ----          23May2010, 02Jul2010, 30Sep2010, 07Oct2010, 23Jan2011
// ----
// ----
// ----

#include "tksynth.h"

#include "SyModuleOscTriPure.h"


void SyModuleOscTriPure::init(void) {
   SyModule::init();

   allocInputs(4);
   addInput("volume", SYPORT_MODE_BE, -4.0f,    4.0f,   1.0f);  // #0
   addInput("pitch",  SYPORT_MODE_BE,  5.0f, 2000.0f, 440.0f);  // #1
   addInput("phase",  SYPORT_MODE_BE,  0.0f,    1.0f,   0.0f);  // #2
   addInput("sync",   SYPORT_MODE_SE,  0.0f,    1.0f,   0.0f);  // #3

   allocOutputs(2);
   addOutput("smp",  SYPORT_MODE_SE);  // #0
   addOutput("sync", SYPORT_MODE_SE);  // #1
}

void SyModuleOscTriPure::reset(void) {
   SyModule::reset();

   angle = 0.0;
}

void SyModuleOscTriPure::doCalc(sUI _framesAdvance) {

   SyInput *in_volume = inputs[0];
   SyInput *in_pitch  = inputs[1];
   SyInput *in_phase  = inputs[2];
   SyInput *in_sync   = inputs[3];

   SyOutput *output   = outputs[0];
   SyOutput *out_sync = outputs[1];

   // Oscillator sync
   sF32 sync = Dsyctl(in_sync);
   if(b_synczerox)
   {
      //printf("xxx csign=%u clast=%u\n", Dsyfltsign(sync), Dsyfltsign(Dsylastctl(in_sync)));
      if(Dsyfltsign(sync) ^ Dsyfltsign(Dsylastctl(in_sync)))
      {
         angle = 0.0f;
      }
   }
   else
   {
      if(sync > 0.0f)
      {
         angle = 0.0f;
      }
   }

   // Get/wrap phase
   sF32 a = (sF32) angle + Dsyctl(in_phase);
   while(a >= 1.0f)
      a -= 1.0f;
   while(a < 0.0f)
      a += 1.0f;

   // Calc output value
   sF32 out;

   if(a < 0.5)
      out = -1.0f + 4.0f * a;
   else
      out =  1.0f - 4.0f * (a - 0.5f);

   out *= Dsyctl(in_volume);
   
   Dsyoutmono(output, out);
   ////output->current_value = out * output->level * in_volume->current_value + output->offset;

   sF64 w = (mapOscPitch(Dsyctl(in_pitch)) / mix_rate);

   angle += w * _framesAdvance;

   while(angle >= 1.0f)
   {
      angle -= 1.0f;
      sync = 1.0f;
   }
   while(angle < 0)
   {
      angle += 1.0f;
      sync = 1.0f;
   }

   Dsyoutmono(out_sync, sync);
}
