// ----
// ---- file   : SyModuleOscTriPureOversample.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2011 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2009, 
// ----          23May2010, 02Jul2010, 30Sep2010, 07Oct2010, 23Jan2011
// ----
// ----
// ----

#include "tksynth.h"

#include "SyModuleOscTriPureOversample.h"


void SyModuleOscTriPureOversample::init(void) {
   SyModule::init();

   allocInputs(4);
   addInput("volume", SYPORT_MODE_BE, -4.0f,    4.0f,   1.0f);  // #0
   addInput("pitch",  SYPORT_MODE_BE,  5.0f, 2000.0f, 440.0f);  // #1
   addInput("phase",  SYPORT_MODE_BE,  0.0f,    1.0f,   0.0f);  // #2
   addInput("sync",   SYPORT_MODE_SE,  0.0f,    1.0f,   0.0f);  // #3

   allocOutputs(2);
   addOutput("smp",  SYPORT_MODE_SE); // #0
   addOutput("sync", SYPORT_MODE_SE); // #1
}

void SyModuleOscTriPureOversample::reset(void) {
   SyModule::reset();

   angle = 0.0;
}

void SyModuleOscTriPureOversample::doCalc(sUI _framesAdvance) {

   SyInput *in_volume = inputs[0];
   SyInput *in_pitch  = inputs[1];
   SyInput *in_phase  = inputs[2];
   SyInput *in_sync   = inputs[3];

   SyOutput *output   = outputs[0];
   SyOutput *out_sync = outputs[1];

   // Oscillator sync
   sF32 sync = Dsyctl(in_sync);
   if(b_synczerox)
   {
      if(Dsyfltsign(sync) ^ Dsyfltsign(Dsylastctl(in_sync)))
      {
         angle = 0.0f;
      }
   }
   else
   {
      if(sync > 0.0f)
      {
         angle = 0.0f;
      }
   }

   // Get/wrap phase
   sF64 a = angle + Dsyctl(in_phase);

#define OVERSAMPLE 16

   sF64 w = (1.0/OVERSAMPLE) * (mapOscPitch(Dsyctl(in_pitch)) / mix_rate);
   
   sF64 r = 0.0;
   for(int i=0; i<OVERSAMPLE; i++)
   {
      while(a >= 1.0)
         a -= 1.0;
      while(a < 0.0)
         a += 1.0;

      if(a < 0.5)
         r += (1.0/OVERSAMPLE) * (-1.0f + 4.0f * a);
      else
         r += (1.0/OVERSAMPLE) * (1.0f - 4.0f * (a - 0.5f));

      a += w;
   }

   r *= Dsyctl(in_volume);

   Dsyoutmono(output, r);
   ////output->current_value = (sF32)(r * in_volume->current_value * output->level + output->offset);

   // tick
   w = (mapOscPitch(Dsyctl(in_pitch)) / mix_rate);

   angle += w * _framesAdvance;

   while(angle >= 1.0f)
   {
      angle -= 1.0f;
      sync = 1.0f;
   }
   while(angle < 0.0f)
   {
      angle += 1.0f;
      sync = 1.0f;
   }

   Dsyoutmono(out_sync, sync);
}
