// ----
// ---- file   : SyModuleOutFloatArray.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2009, 
// ----          23May2010, 30Sep2010, 07Oct2010
// ----
// ----
// ----

#define YAC_EPSILONCOMPARE_ABS defined
#define YAC_EPSILONCOMPARE_ABS_DEFAULT defined

#include "tksynth.h"

#include "SyModuleOutFloatArray.h"


SyModuleOutFloatArray::SyModuleOutFloatArray(void) {
   num_iterations = 1;
   level          = 1.0f;
   b_blockmode    = 0;
}

SyModuleOutFloatArray::~SyModuleOutFloatArray() {
}


void SyModuleOutFloatArray::init(void) {
   SyModule::init();

   allocInputs(1);
   addInput("smp", SYPORT_MODE_SE, -1.0f, 1.0f, 0.0f); // #0

   SyInput *inSmp = (SyInput *) getInputById(0);
   ////printf("xxx inSmp=0x%p num_inputs=%u\n", inSmp, num_inputs);
   inSmp->setNumChannels(2);
   inSmp->setMinResetValue(-1.0f);
   inSmp->setMaxResetValue( 1.0f);

   ////printf("xxx SyModuleOutFloatArray num_inputs=%d\n", num_inputs);
}

void SyModuleOutFloatArray::setLevel(sF32 _level) {
   level = _level;
}

sF32 SyModuleOutFloatArray::getLevel(void) {
   return level;
}

void SyModuleOutFloatArray::setNumIterations(sUI _num) {
   if(_num > 0)
   {
      num_iterations = _num;
   }
   else
   {
      num_iterations = 1;
   }
}

sUI SyModuleOutFloatArray::getNumIterations(void) {
   return num_iterations;
}

void SyModuleOutFloatArray::setEnableBlockMode(sSI _bEnabled) {
   b_blockmode = _bEnabled;
}

sSI SyModuleOutFloatArray::getEnableBlockMode(void) {
   return b_blockmode;
}

void SyModuleOutFloatArray::calcStereoFloatArray(YAC_Object *_fa) {
   ////printf("xxx 0 calcFloatArray: --------------------------------------------- b_blockmode=%d\n", b_blockmode);
   
   if(YAC_BCHK(_fa, YAC_CLID_FLOATARRAY))
   {
      YAC_CAST_ARG(YAC_FloatArray, fa, _fa);

      sUI num = fa->num_elements / 2;

      if(b_blockmode)
      {
         calcBlock(next_tick_count, num);

         SyInput *input = inputs[0];

         sF32 *d = fa->elements;
         
         if(2 == input->num_channels)
         {
            if(Dfltequal(level, 1.0f))
            {
               // Copy stereo input to output buffer
               memcpy(d, input->buffer, sizeof(sF32)*2*num);
            }
            else
            {
               // Copy scaled stereo input to output buffer
               sUI i;
               sF32 *s = input->buffer;
               for(i=0; i<num; i++)
               {
                  ////printf("xxx in[%u] = (%f, %f)\n", i, s[0], s[1]);
                  *d++ = *s++ * level;
                  *d++ = *s++ * level;
               }
            }
         }
         else
         {
            // Copy mono input to output buffer
            sUI i;
            sF32 *s = input->buffer;
            for(i=0; i<num; i++)
            {
               sF32 t = *s++;
               *d++ = t;
               *d++ = t;
            }
         }

         next_tick_count++;
      }
      else
      {
         // Sample-exact calculation 
         //  (quite slow but allows for some interesting feedback effects)
         //  (module parameters are also evaluated per-sample rather than per-block)
         sUI i;
         sUI k = 0;
         for(i=0; i<num; i++)
         {
            //printf("xxx calcFloatArray: --------------------------------------------- i=%d/%d\n", i, num);
            sUI j;
            for(j=0; j<num_iterations; j++)
            {
               calc(next_tick_count, 1);
            }

            if(2 == inputs[0]->num_channels) // (todo) #input ch should never matter, num_channels should only be configurable per -output-
            {
               // Stereo
               fa->elements[k++] = inputs[0]->current_value[0] * level;
               fa->elements[k++] = inputs[0]->current_value[1] * level;
            }
            else
            {
               // Mono to stereo
               sF32 v = inputs[0]->current_value[0] * level;
               fa->elements[k++] = v;
               fa->elements[k++] = v;
            }

            next_tick_count++;
         }
      }

      ////printf("xxx END calcFloatArray: ---------------------------------------------\n");

   }

}
