// ----
// ---- file   : SyModuleRevFreeverb.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 22Jun2010, 30Sep2010, 07Oct2010
// ----
// ----
// ----

#define YAC_EPSILONCOMPARE_ABS defined
#define YAC_EPSILONCOMPARE_ABS_DEFAULT defined

#include "tksynth.h"

#include "SyModuleRevFreeverb.h"


SyModuleRevFreeverb::SyModuleRevFreeverb(void) {
   rev = new revmodel();
}

SyModuleRevFreeverb::~SyModuleRevFreeverb() {
   delete rev;
   rev = NULL;
}

void SyModuleRevFreeverb::init(void) {
   SyModule::init();

   allocInputs(7);
   addInput("smp",      SYPORT_MODE_SE, -1.0f, 1.0f, 0.0f);   // #0
   addInput("roomsize", SYPORT_MODE_BE,  0.0f, 1.0f, 0.89f);  // #1
   addInput("damp",     SYPORT_MODE_BE,  0.0f, 1.0f, 0.85f);  // #2
   addInput("wet",      SYPORT_MODE_BE,  0.0f, 1.0f, 0.1f);   // #3
   addInput("dry",      SYPORT_MODE_BE,  0.0f, 1.0f, 0.4f);   // #4
   addInput("width",    SYPORT_MODE_BE,  0.0f, 1.0f, 0.8f);   // #5
   addInput("mode",     SYPORT_MODE_BE,  0.0f, 1.0f, 0.0f);   // #6 

   allocOutputs(1);
   addOutput("smp", SYPORT_MODE_SE);  // #0
}

void SyModuleRevFreeverb::reset(void) {
   SyModule::reset();

   rev->mute();
   // printf("xxx SyModuleRevFreeverb::reset\n");
}

void SyModuleRevFreeverb::doCalc(sUI _framesAdvance) {

   SyInput *in_smp      = inputs[0];
   SyInput *in_roomsize = inputs[1];
   SyInput *in_damp     = inputs[2];
   SyInput *in_wet      = inputs[3];
   SyInput *in_dry      = inputs[4];
   SyInput *in_width    = inputs[5];
   SyInput *in_mode     = inputs[6];

   SyOutput *output = outputs[0];

   rev->setroomsize (Dsyctl(in_roomsize));
   rev->setdamp     (Dsyctl(in_damp));
   rev->setwet      (Dsyctl(in_wet));
   rev->setdry      (Dsyctl(in_dry));
   rev->setwidth    (Dsyctl(in_width));
   rev->setmode     (Dsyctl(in_mode));

   sF32 *inL, *inR;
   sF32 out[2];

   out[0] = out[1] = 0.0f;

   inL = &in_smp->current_value[0];

   if(2 == in_smp->num_channels)
   {
      // Stereo input
      inR = &in_smp->current_value[1];
   }
   else
   {
      // Mono input
      inR = inL;
   }

   // Calculate reverb
   rev->processmix(inL, inR,
                   out, out+1,
                   1, // #samples
                   0  // skip
                   );

   // Scale/offset output samples and copy to output port
   for(sUI chIdx=0; chIdx<2; chIdx++)
   {
      Dsyout(output, out[chIdx]);
   }

}

void SyModuleRevFreeverb::doCalcBlock(sUI _numFrames) {

   SyInput *in_smp      = inputs[0];
   SyInput *in_roomsize = inputs[1];
   SyInput *in_damp     = inputs[2];
   SyInput *in_wet      = inputs[3];
   SyInput *in_dry      = inputs[4];
   SyInput *in_width    = inputs[5];
   SyInput *in_mode     = inputs[6];

   SyOutput *output = outputs[0];

   if(
      (             2 != in_smp->num_channels)  ||
      (             2 != output->num_channels)  ||
      (SYPORT_MODE_BE != in_roomsize->mode)     ||
      (SYPORT_MODE_BE != in_damp->mode)         ||
      (SYPORT_MODE_BE != in_wet->mode)          ||
      (SYPORT_MODE_BE != in_dry->mode)          ||
      (SYPORT_MODE_BE != in_width->mode)        ||
      (SYPORT_MODE_BE != in_mode->mode) 
      )
   {
      SyModule::doCalcBlock(_numFrames);
      return;
   }

   rev->setroomsize (Dsyctlbm(in_roomsize));
   rev->setdamp     (Dsyctlbm(in_damp));
   rev->setwet      (Dsyctlbm(in_wet));
   rev->setdry      (Dsyctlbm(in_dry));
   rev->setwidth    (Dsyctlbm(in_width));
   rev->setmode     (Dsyctlbm(in_mode));

   // Calculate reverb

   ////memset(output->buffer, 0, sizeof(sF32) * _numFrames * 2);

   rev->processreplace(in_smp->buffer, in_smp->buffer+1,
                       output->buffer, output->buffer+1,
                       _numFrames, // #samples
                       2  // skip (interleaved L/R pairs)
                       );

   if(! (Dfltequal(output->level, 1.0f) && Dfltequal(output->offset, 0.0f)) )
   {
      // Special case: scale/offset output values
      sUI i;
      sF32 *s = output->buffer;
      for(i=0; i<_numFrames; i++)
      {
         *s++ = *s * output->level + output->offset; // L
         *s++ = *s * output->level + output->offset; // R
      }
   }

}
