// ----
// ---- file   : SyModuleSigQuant.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2013 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 30May2013
// ----
// ----
// ----

#include "tksynth.h"

#include "SyModuleSigQuant.h"


void SyModuleSigQuant::init(void) {
   SyModule::init();

   allocInputs(3);
   addInput("smp",    SYPORT_MODE_SE, -1.0f,  1.0f, 0.0f);      // #0
   addInput("quant",  SYPORT_MODE_BE, -1.0f,  1.0f, 1.0f/256);  // #1
   addInput("offset", SYPORT_MODE_BE, -1.0f,  1.0f, 0.0f);      // #2

   allocOutputs(1);
   addOutput("smp", SYPORT_MODE_SE); // #0
}

void SyModuleSigQuant::reset(void) {
   SyModule::reset();
}

void SyModuleSigQuant::doCalc(sUI _framesAdvance) {

   SyInput *in_smp     = inputs[0];
   SyInput *in_quant   = inputs[1];
   SyInput *in_offset  = inputs[2];

   SyOutput *out_smp = outputs[0];

   sF32 quant  = Dsyctl(in_quant);
   sF32 offset = Dsyctl(in_offset);

   Dsyloopch(out_smp)
   {
      sF32 smp = Dsyval(in_smp) - offset;

      if(quant != 0)
      {
         smp = ((sSI)(smp / quant)) * quant;
      }

      sF32 out = smp + offset;

      Dsyout(out_smp, out);
   }
}
