// ----
// ---- file   : SyModuleWSHardBounce.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2009, 
// ----          23May2010, 30Sep2010, 07Oct2010
// ----
// ----
// ----

#include "tksynth.h"

#include "SyModuleWSHardBounce.h"


void SyModuleWSHardBounce::init(void) {
   SyModule::init();

   allocInputs(3);
   addInput("smp",     SYPORT_MODE_SE, -1.0f,  1.0f, 0.0f); // #0
   addInput("drive",   SYPORT_MODE_BE,  1.0f, 16.0f, 1.0f); // #1
   addInput("damping", SYPORT_MODE_BE,  0.0f,  1.0f, 1.0f); // #2

   allocOutputs(1);
   addOutput("smp", SYPORT_MODE_SE); // #0
}

void SyModuleWSHardBounce::reset(void) {
   SyModule::reset();
}

void SyModuleWSHardBounce::doCalc(sUI _framesAdvance) {

   SyInput *in_smp      = inputs[0];
   SyInput *in_drive    = inputs[1];
   SyInput *in_damping  = inputs[2];
   SyOutput *out_smp = outputs[0];

   sF64 dmp = Dsyctl(in_damping);
   if(dmp < 0)
      dmp = 0;
   else if(dmp > 1)
      dmp = 1;

   Dsyloopch(in_smp)
   {
      sF64 out = Dsyval(in_smp) * Dsyctl(in_drive);
      while( (out>1.0) ||(out<-1.0) )
      {
         if(out > 1.0)
            out = 1.0 - dmp*(out-1.0);
         if(out < -1.0)
            out = -1.0 + dmp*(-1.0 - out);
      }

      Dsyout(out_smp, out);
      ////out_smp->current_value = (sF32) (out * out_smp->level + out_smp->offset);
   }
}
