// ----
// ---- file   : SyModuleWSRingModulator.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 01Jul2010, 30Sep2010, 07Oct2010
// ----
// ----
// ----

#include "tksynth.h"

#include "SyModuleWSRingModulator.h"


void SyModuleWSRingModulator::init(void) {
   SyModule::init();

   allocInputs(3);
   addInput("smp",    SYPORT_MODE_SE, -1.0f, 1.0f, 0.0f); // #0
   addInput("smp2",   SYPORT_MODE_SE, -1.0f, 1.0f, 0.0f); // #1
   addInput("amount", SYPORT_MODE_BE,  0.0f, 1.0f, 1.0f); // #2

   allocOutputs(1);
   addOutput("smp", SYPORT_MODE_SE); // #0
}

void SyModuleWSRingModulator::reset(void) {
   SyModule::reset();
}

void SyModuleWSRingModulator::doCalc(sUI _framesAdvance) {

   SyInput *in_smp    = inputs[0];
   SyInput *in_smp2   = inputs[1];
   SyInput *in_amount = inputs[2];

   SyOutput *out_smp = outputs[0];

   sF32 amount = Dsyclamp(Dsyctl(in_amount), 0.0f, 1.0f);

   Dsyloopch(in_smp)
   {
      sF64 inSmp = Dsyval(in_smp);
      sF64 out =  inSmp * Dsyval(in_smp2);
      out = (out * amount) + inSmp * (1.0f - amount);

      Dsyout(out_smp, out);
   }
}

void SyModuleWSRingModulator::doCalcBlock(sUI _numFrames) {

   SyInput *in_smp    = inputs[0];
   SyInput *in_smp2   = inputs[1];
   SyInput *in_amount = inputs[2];

   SyOutput *out_smp = outputs[0];

   if(SYPORT_MODE_BE == in_amount->mode)
   {
      sF32 amount = Dsyclamp(in_amount->buffer[0], 0.0f, 1.0f);

      if(2 == in_smp->num_channels)
      {
         if(2 == in_smp2->num_channels)
         {
            const sF32 *pInSmp  = in_smp->buffer;
            const sF32 *pInSmp2 = in_smp2->buffer;
            sF32 *d = out_smp->buffer;

            sUI i;
            const sUI numSamples = 2 * _numFrames;
            for(i=0; i<numSamples; i++)
            {
               d[i] = (pInSmp[i] * pInSmp2[i] * amount) + pInSmp[i] * (1.0f - amount);
            }
            return;
         }
      }
   }

   // Fallback to default implementation
   SyModule::doCalcBlock(_numFrames);
}
