// ----
// ---- file   : SyPort.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2010 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2090, 
// ----          22May2010, 30Sep2010
// ----
// ----
// ----

#include "tksynth.h"


SyPort::SyPort(void) {
   name   = NULL;
   level  = 1.0f;
   offset = 0.0f;

   min_level  = 0.0f;
   max_level  = 2.0f;
   min_offset = -1.0f;
   max_offset = 1.0f;

   buffer = new sF32[SYPORT_MAXBUFSIZE * SYPORT_MAXCHANNELS];
   memset(buffer, 0, SYPORT_MAXBUFSIZE * SYPORT_MAXCHANNELS);

   for(sUI chIdx=0; chIdx < SYPORT_MAXCHANNELS; chIdx++)
   {
      current_value[chIdx] = 0.0f;
   }
   
   num_channels = 1;

   default_mode = mode = SYPORT_MODE_SE; // will be override by addInput() resp. addOutput() in SyModule
}

SyPort::~SyPort() {
   YAC_DELETE_SAFE(name);

   delete [] buffer;
   buffer = NULL;
}

YAC_Object *SyPort::getModule(void) {
   return module;
}

YAC_Object *SyPort::getName(void) {
   return name;
}

void SyPort::setName(YAC_Object *_name) {
   if(NULL == name)
   {
      name = YAC_New_String();
   }
   name->yacCopy(_name);
}

void SyPort::setLevel(sF32 _level) {
   level = _level;
}

sF32 SyPort::getLevel(void) {
   return level;
}

void SyPort::setOffset(sF32 _offset) {
   offset = _offset;
}

sF32 SyPort::getOffset(void) {
   return offset;
}

void SyPort::setMinLevel(sF32 _min) {
   min_level = _min;
}

sF32 SyPort::getMinLevel(void) {
   return min_level;
}

void SyPort::setMaxLevel(sF32 _max) {
   max_level = _max;
}

sF32 SyPort::getMaxLevel(void) {
   return max_level;
}

void SyPort::setMinOffset(sF32 _min) {
   min_offset = _min;
}

sF32 SyPort::getMinOffset(void) {
   return min_offset;
}

void SyPort::setMaxOffset(sF32 _max) {
   max_offset = _max;
}

sF32 SyPort::getMaxOffset(void) {
   return max_offset;
}

void SyPort::setNumChannels(sUI _numChannels) {
   if(_numChannels < SYPORT_MAXCHANNELS)
   {
      num_channels = _numChannels;
   }
   else
   {
      num_channels = SYPORT_MAXCHANNELS;
   }
}

sUI SyPort::getNumChannels(void) {
   return num_channels;
}

void SyPort::setMode(sUI _mode) {
   mode = _mode;
}

sUI SyPort::getMode(void) {
   return mode;
}
