// ----
// ---- file   : SyPort.h
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2011 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2009, 
// ----          22May2010, 23May2010, 30Sep2010, 19Jan2011
// ----
// ----
// ----

#ifndef __TKSYNTH_PORT_H__
#define __TKSYNTH_PORT_H__

YG("tksynth");

class SyModule;

#define SYPORT_MAXCHANNELS 2
//#define SYPORT_MAXBUFSIZE  256
#define SYPORT_MAXBUFSIZE  4096


YCR class SyPort : public YAC_Object {
  public:
   YAC_String *name; // deletable name object
   sF32 level;
   sF32 offset;

   SyModule *module;

   // sample exact (#blocksize samples per block
#define SYPORT_MODE_SE YCI 0

   // block exact (one sample per block)
#define SYPORT_MODE_BE YCI 1

   sUI mode;         // initial configuration set by module 
   sUI default_mode; // initial configuration set by module

   sF32 *buffer; // used in non-feedback mode

   sF32 current_value[SYPORT_MAXCHANNELS]; // new calculated value(s) (used in feedback mode)

   sUI num_channels; // mono(1) or stereo(2)

   sF32 min_level;  // suggested value (*not* clamped, just a hint for the GUI)
   sF32 max_level;  // ^^
   sF32 min_offset; // ^^ 
   sF32 max_offset; // ^^

  public:
   YAC_POOLED(SyPort, YAC_POOL_PRIORITY_LOW);

   SyPort(void);
   ~SyPort();

   YM YAC_Object *getName (void);
   YM void        setName (YAC_Object *_name); // for IO modules

   YM void setLevel  (sF32 _level);
   YM sF32 getLevel  (void);

   YM void setOffset (sF32 _offset);
   YM sF32 getOffset (void);

   YM void setMinLevel (sF32 _min);
   YM sF32 getMinLevel (void);

   YM void setMaxLevel (sF32 _max);
   YM sF32 getMaxLevel (void);

   YM void setMinOffset (sF32 _min);
   YM sF32 getMinOffset (void);

   YM void setMaxOffset (sF32 _max);
   YM sF32 getMaxOffset (void);

   YM void setNumChannels (sUI _num);
   YM sUI  getNumChannels (void);

   YM void setMode (sUI _mode);
   YM sUI  getMode (void);


   YM YAC_Object *getModule (void);
};

#define _SyPort SyPort


#endif // __TKSYNTH_PORT_H__
