// ----
// ---- file   : tksynth.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2013 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2009, 
// ----          22May2010, 13Jun2010, 15Jun2010, 22Jun2010, 23Jun2010, 29Jun2010, 01Jul2010
// ----          22Sep2010, 28May2013, 29May2013, 30May2013
// ----
// ----
// ----

#include "tksynth.h"

#include "SyModuleOscSinePure.h"
#include "SyModuleOscPulsePure.h"
#include "SyModuleOscPulsePureOversample.h"
#include "SyModuleOscTriPure.h"
#include "SyModuleOscTriPureOversample.h"
#include "SyModuleOscSawPure.h"
#include "SyModuleOscSawDSFBLIT.h"
#include "SyModuleOscSawPureOversample.h"
#include "SyModuleOscNoise1.h"
#include "SyModuleOscSHI.h"

#include "SyModuleFltBiquadRBJ.h"
#include "SyModuleFltMoog1.h"
#include "SyModuleFlt303.h"
#include "SyModuleFltPhaser6St.h"

#include "SyModuleEQ3.h"

#include "SyModuleWSTanh.h"
#include "SyModuleWSTanhAsym.h"
#include "SyModuleWSSmoothStep.h"
#include "SyModuleWSSin.h"
#include "SyModuleWSHardClip.h"
#include "SyModuleWSHardBounce.h"
#include "SyModuleWSRingModulator.h"

#include "SyModuleOutFloatArray.h"

#include "SyModuleExtSample.h"

#include "SyModuleDlySimple.h"
#include "SyModuleDlyModulate.h"

#include "SyModuleRevFreeverb.h"

#include "SyModuleDynCompressor.h"

#include "SyModuleChnStereoSplit.h"
#include "SyModuleChnStereoJoin.h"

#include "SyModuleOpMixMax.h"
#include "SyModuleOpMul.h"
#include "SyModuleOpMin.h"
#include "SyModuleOpMax.h"
#include "SyModuleOpAbsMin.h"
#include "SyModuleOpAbsMax.h"
#include "SyModuleOpPower.h"
#include "SyModuleOpMulAsym.h"

#include "SyModuleSigQuant.h"

#include "SyModuleIO.h"


YAC_Host *yac_host;

////#include "tksynth_ext_refs.cpp"

#include "ying_tksynth_SyModuleIO.cpp"

#include "ying_tksynth_SyModuleOsc.cpp"
#include "ying_tksynth_SyModuleOscSinePure.cpp"
#include "ying_tksynth_SyModuleOscPulsePure.cpp"
#include "ying_tksynth_SyModuleOscPulsePureOversample.cpp"
#include "ying_tksynth_SyModuleOscTriPure.cpp"
#include "ying_tksynth_SyModuleOscTriPureOversample.cpp"
#include "ying_tksynth_SyModuleOscSawPure.cpp"
#include "ying_tksynth_SyModuleOscSawDSFBLIT.cpp"
#include "ying_tksynth_SyModuleOscSawPureOversample.cpp"
#include "ying_tksynth_SyModuleOscNoise1.cpp"
#include "ying_tksynth_SyModuleOscSHI.cpp"

#include "ying_tksynth_SyModuleFltBiquadRBJ.cpp"
#include "ying_tksynth_SyModuleFltMoog1.cpp"
#include "ying_tksynth_SyModuleFlt303.cpp"
#include "ying_tksynth_SyModuleFltPhaser6St.cpp"

#include "ying_tksynth_SyModuleEQ3.cpp"

#include "ying_tksynth_SyModuleWSTanh.cpp"
#include "ying_tksynth_SyModuleWSTanhAsym.cpp"
#include "ying_tksynth_SyModuleWSSmoothStep.cpp"
#include "ying_tksynth_SyModuleWSSin.cpp"
#include "ying_tksynth_SyModuleWSHardClip.cpp"
#include "ying_tksynth_SyModuleWSHardBounce.cpp"
#include "ying_tksynth_SyModuleWSRingModulator.cpp"

#include "ying_tksynth_SyModuleOutFloatArray.cpp"

#include "ying_tksynth_SyModuleExtSample.cpp"

#include "ying_tksynth_SyModuleDlySimple.cpp"
#include "ying_tksynth_SyModuleDlyModulate.cpp"

#include "ying_tksynth_SyModuleRevFreeverb.cpp"

#include "ying_tksynth_SyModuleDynCompressor.cpp"

#include "ying_tksynth_SyModuleChnStereoSplit.cpp"
#include "ying_tksynth_SyModuleChnStereoJoin.cpp"

#include "ying_tksynth_SyModuleOpMixMax.cpp"
#include "ying_tksynth_SyModuleOpMul.cpp"
#include "ying_tksynth_SyModuleOpMin.cpp"
#include "ying_tksynth_SyModuleOpMax.cpp"
#include "ying_tksynth_SyModuleOpAbsMin.cpp"
#include "ying_tksynth_SyModuleOpAbsMax.cpp"
#include "ying_tksynth_SyModuleOpPower.cpp"
#include "ying_tksynth_SyModuleOpMulAsym.cpp"

#include "ying_tksynth_SyModuleSigQuant.cpp"

#include "ying_tksynth.cpp"


// Implement standard exception ID variables (see yac.h)
Dyac_std_exid_impl;

YAC_MutexHandle mtx_mod;


// ---------------------------------------------------------------------------- YAC_Init
void YAC_CALL YAC_Init(YAC_Host *_host) {
	yac_host=_host;

   // Resolve "standard" exception IDs
   Dyac_std_exid_resolve;

   YAC_Init_tksynth(_host);

   // Inheritance of exported classes
   yac_host->cpp_typecast_map[clid_SyInput]                 [clid_SyPort] = 1;
   yac_host->cpp_typecast_map[clid_SyOutput]                [clid_SyPort] = 1;

   yac_host->cpp_typecast_map[clid_SyModuleOsc]                     [clid_SyModule]    = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscSinePure]             [clid_SyModule]    = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscSinePure]             [clid_SyModuleOsc] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscPulsePure]            [clid_SyModule]    = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscPulsePure]            [clid_SyModuleOsc] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscPulsePureOversample]  [clid_SyModule]    = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscPulsePureOversample]  [clid_SyModuleOsc] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscTriPure]              [clid_SyModule]    = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscTriPure]              [clid_SyModuleOsc] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscTriPureOversample]    [clid_SyModule]    = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscTriPureOversample]    [clid_SyModuleOsc] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscSawPure]              [clid_SyModule]    = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscSawPure]              [clid_SyModuleOsc] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscSawDSFBLIT]           [clid_SyModule]    = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscSawDSFBLIT]           [clid_SyModuleOsc] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscSawPureOversample]    [clid_SyModule]    = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscSawPureOversample]    [clid_SyModuleOsc] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscNoise1]               [clid_SyModule]    = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscNoise1]               [clid_SyModuleOsc] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscSHI]                  [clid_SyModule]    = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOscSHI]                  [clid_SyModuleOsc] = 1;

   yac_host->cpp_typecast_map[clid_SyModuleFltBiquadRBJ]    [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleFltMoog1]        [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleFlt303]          [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleFltPhaser6St]    [clid_SyModule] = 1;

   yac_host->cpp_typecast_map[clid_SyModuleEQ3]             [clid_SyModule] = 1;

   yac_host->cpp_typecast_map[clid_SyModuleWSTanh]          [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleWSTanhAsym]      [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleWSSmoothStep]    [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleWSSin]           [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleWSHardClip]      [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleWSHardBounce]    [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleWSRingModulator] [clid_SyModule] = 1;

   yac_host->cpp_typecast_map[clid_SyModuleIO]              [clid_SyModule] = 1;

   yac_host->cpp_typecast_map[clid_SyModuleOutFloatArray]   [clid_SyModule] = 1;

   yac_host->cpp_typecast_map[clid_SyModuleExtSample]       [clid_SyModule] = 1;

   yac_host->cpp_typecast_map[clid_SyModuleDlySimple]       [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleDlyModulate]     [clid_SyModule] = 1;

   yac_host->cpp_typecast_map[clid_SyModuleRevFreeverb]     [clid_SyModule] = 1;

   yac_host->cpp_typecast_map[clid_SyModuleDynCompressor]   [clid_SyModule] = 1;

   yac_host->cpp_typecast_map[clid_SyModuleChnStereoSplit]  [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleChnStereoJoin]   [clid_SyModule] = 1;

   yac_host->cpp_typecast_map[clid_SyModuleOpMixMax]        [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOpMul]           [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOpMin]           [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOpMax]           [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOpAbsMin]        [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOpAbsMax]        [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOpPower]         [clid_SyModule] = 1;
   yac_host->cpp_typecast_map[clid_SyModuleOpMulAsym]       [clid_SyModule] = 1;

   yac_host->cpp_typecast_map[clid_SyModuleSigQuant]        [clid_SyModule] = 1;

   mtx_mod = yac_host->yacMutexCreate();
}

void YAC_CALL YAC_Exit(YAC_Host *_host) {
   YAC_Exit_tksynth(_host);

   yac_host->yacMutexDestroy(mtx_mod);
}


#include <yac_host.cpp>
