// ----
// ---- file   : tksynth.h
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2013 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : This is part of the "syntracker" midi/audio sequencer.
// ----
// ---- changed: 10May2009, ..., 08Jun2009, 
// ----          22May2010, 13Jun2010, 15Jun2010, 02Jul2010, 30Sep2010, 07Oct2010, 23Jan2011
// ----          28May2013
// ----
// ----
// ----

#ifndef __TKSYNTH_H__
#define __TKSYNTH_H__

#include <stdio.h>
#include <stdarg.h>
#include <math.h>

//#define YAC_BIGSTRING
////#define YAC_PRINTF (already defined in yac.h)
#include <yac.h>

// Convert milliseconds to #frames
#define TKSYNTH_MILLISEC_TO_FRAMES(msec) ((sSI)(msec * mix_rate / 1000.0f))

#define TKSYNTH_MILLISEC_ADV_TO_FRAMES(msec, adv) ((sSI)(msec * mix_rate / (adv * 1000.0f)))


// Loop all channel values
#define Dsyloopallch() for(sUI chIdx=0; chIdx<SYPORT_MAXCHANNELS; chIdx++)

// Loop input channel values
#define Dsyloopch(p) for(sUI chIdx=0; chIdx < (p)->num_channels; chIdx++)

// Read (mono) controller value from input <p>
#define Dsyctl(p) (p)->current_value[0]

// Read (mono) value from input <p>
#define Dsymono(p) (p)->current_value[0]

// Read (mono) controller buffer sample from input <p>
#define Dsyctlbm(p) (p)->buffer[0]

// Read last/previous (mono) controller value from input <p>
#define Dsylastctl(p) (p)->last_value[0]

// Read current channel controller value from <p>  (to be used with Dsyloopch)
#define Dsyval(p) (p)->current_value[chIdx]

// Write scaled/offset output value (current channel)
#define Dsyout(p, o) (p)->current_value[chIdx] = (sF32) (o  * (p)->level + (p)->offset)

// Write scaled/offset output value (mono)
#define Dsyoutmono(p, o) (p)->current_value[0] = (sF32) (o  * (p)->level + (p)->offset)

// Write scaled/offset output value (left channel)
#define Dsyoutleft(p, o) (p)->current_value[0] = (sF32) (o  * (p)->level + (p)->offset)

// Write scaled/offset output value (right channel)
#define Dsyoutright(p, o) (p)->current_value[1] = (sF32) (o  * (p)->level + (p)->offset)

// Declare/get channel state object
#define Dsystate(clazz) clazz *st = &state[chIdx]

// Clamp float value to given range
#define Dsyclamp(v, min, max) (((v) < (min)) ? (min) : ((v) > (max)) ? (max) : (v))

// Wrap float value around given range
#define Dsywrap(v, min, max) (((v) < (min)) ? (((v)-(min))+(max)) : ((v) > (max)) ? (((v)-(max))+(min)) : (v))

// Get sign of float
#define Dsyfltsign(a) ( (*(sUI*)&(a)) >> 31 )


#include "SyPort.h"
#include "SyInput.h"
#include "SyOutput.h"
#include "SyConnection.h"
#include "SyModule.h"
#include "SyModuleOsc.h"

////#include "SyModuleOutFloatArray.h"

////#include "SyModuleExtSample.h"



#include "ying_tksynth.h"


#endif // __TKSYNTH_H__
